@@ -36,7 +36,7 @@ using Mono.Cecil;
 
 namespace Gendarme.Framework.Rocks {
 
-	// add CustomAttribute[Collection] extensions methods here
+	// add Method[Reference|Definition][Collection] extensions methods here
 	// only if:
 	// * you supply minimal documentation for them (xml)
 	// * you supply unit tests for them
@@ -69,7 +69,7 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the method is a finalizer, False otherwise.</returns>
 		public static bool IsFinalizer (this MethodReference self)
 		{
-			return (self.HasThis && (self.Parameters.Count == 0) && (self.Name == "Finalize") &&
+			return (self.HasThis && !self.HasParameters && (self.Name == "Finalize") &&
 				(self.ReturnType.ReturnType.FullName == "System.Void"));
 		}
 
@@ -82,10 +82,11 @@ namespace Gendarme.Framework.Rocks {
 		public static bool IsGeneratedCode (this MethodReference self)
 		{
 			MethodDefinition method = self.Resolve ();
-			if (method.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
-				return true;
-
-			return method.DeclaringType.IsGeneratedCode ();
+			if ((method != null) && method.HasCustomAttributes) {
+				if (method.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
+					return true;
+			}
+			return self.DeclaringType.IsGeneratedCode ();
 		}
 
 		/// <summary>
@@ -118,16 +119,57 @@ namespace Gendarme.Framework.Rocks {
 				return false;
 			}
 
-			switch (method.Parameters.Count) {
-			case 0:
-				// Main (void)
+			// Main (void)
+			if (!method.HasParameters)
 				return true;
-			case 1:
-				// Main (string[] args)
-				return (method.Parameters [0].ParameterType.Name == "String[]");
-			default:
+
+			if (method.Parameters.Count != 1)
 				return false;
+
+			// Main (string[] args)
+			return (method.Parameters [0].ParameterType.Name == "String[]");
+		}
+
+		/// <summary>
+		/// Check if a method is an override to a virtual method of a base type.
+		/// </summary>
+		/// <param name="self">The MethodReference on which the extension method can be called.</param>
+		/// <returns>True if the method is an override to a virtual method, False otherwise</returns>
+		public static bool IsOverride (this MethodReference self)
+		{
+			MethodDefinition method = self.Resolve ();
+			if ((method == null) || !method.IsVirtual)
+				return false;
+
+			TypeDefinition declaring = method.DeclaringType;
+			TypeDefinition parent = declaring.BaseType != null ? declaring.BaseType.Resolve () : null;
+			while (parent != null) {
+				string name = method.Name;
+				string retval = method.ReturnType.ReturnType.FullName;
+				int pcount = method.HasParameters ? method.Parameters.Count : 0;
+				foreach (MethodDefinition md in parent.Methods) {
+					if (name != md.Name)
+						continue;
+					if (retval != md.ReturnType.ReturnType.FullName)
+						continue;
+					if ((md.HasParameters && (pcount == 0)) || (pcount != md.Parameters.Count))
+						continue;
+
+					bool ok = true;
+					for (int i = 0; i < pcount; i++) {
+						if (method.Parameters [i].ParameterType.FullName != md.Parameters [i].ParameterType.FullName) {
+							ok = false;
+							break;
+						}
+					}
+					if (!ok)
+						continue;
+
+					return md.IsVirtual;
+				}
+				parent = parent.BaseType != null ? parent.BaseType.Resolve () : null;
 			}
+			return false;
 		}
 
 		/// <summary>
@@ -137,7 +179,10 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the method is a getter or a setter, False otherwise</returns>
 		public static bool IsProperty (this MethodReference self)
 		{
-			return ((self.Resolve ().SemanticsAttributes & (MethodSemanticsAttributes.Getter | MethodSemanticsAttributes.Setter)) != 0);
+			MethodDefinition method = self.Resolve ();
+			if (method == null)
+				return false;
+			return ((method.SemanticsAttributes & (MethodSemanticsAttributes.Getter | MethodSemanticsAttributes.Setter)) != 0);
 		}
 
 		/// <summary>
@@ -148,24 +193,11 @@ namespace Gendarme.Framework.Rocks {
 		public static bool IsVisible (this MethodReference self)
 		{
 			MethodDefinition method = self.Resolve ();
-			if (method.IsPrivate || method.IsAssembly)
+			if ((method == null) || method.IsPrivate || method.IsAssembly)
 				return false;
 			return self.DeclaringType.Resolve ().IsVisible ();
 		}
 
-		/// <summary>
-		/// Resolve a MethodReference into a MethodDefinition.
-		/// </summary>
-		/// <param name="self">The MethodReference on which the extension method can be called.</param>
-		/// <returns>A MethodDefinition if resolved, null otherwise.</returns>
-		public static MethodDefinition Resolve (this MethodReference self)
-		{
-			MethodDefinition method = (self as MethodDefinition);
-			if (method == null)
-				method = AssemblyResolver.Resolver.Resolve (self);
-			return method;
-		}
-
 		/// <summary>
 		/// Check if the method has the signature of an Event callback.
 		/// They are usually of the form: void Method (object sender, EventArgs ea), where
@@ -175,9 +207,23 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the method has the signature of an event callback.</returns>
 		public static bool IsEventCallback (this MethodReference self)
 		{
-			var parameters = self.Resolve ().Parameters;
+			MethodDefinition method = self.Resolve ();
+			if ((method == null) || !method.HasParameters)
+				return false;
 
-			return parameters.Count == 2 && parameters [1].ParameterType.Inherits ("System.EventArgs");
+			ParameterDefinitionCollection parameters = method.Parameters;
+			if (parameters.Count != 2)
+				return false;
+
+			TypeReference type = parameters [1].ParameterType;
+			GenericParameter gp = (type as GenericParameter);
+			if (gp == null)
+				return type.Inherits ("System.EventArgs");
+
+			if (gp.HasConstraints && (gp.Constraints.Count == 1))
+				return (gp.Constraints [0].FullName == "System.EventArgs");
+
+			return false;
 		}
 	}
 }
