@@ -31,6 +31,7 @@
 // THE SOFTWARE.
 
 using System;
+using System.Collections;
 using System.Collections.Generic;
 
 using Mono.Cecil;
@@ -65,10 +66,41 @@ namespace Gendarme.Framework.Rocks {
 		public static IEnumerable<MethodDefinition> AllMethods (this TypeReference self)
 		{
 			TypeDefinition type = self.Resolve ();
-			foreach (MethodDefinition ctor in type.Constructors)
-				yield return ctor;
-			foreach (MethodDefinition method in type.Methods)
-				yield return method;
+			if (type.HasConstructors) {
+				foreach (MethodDefinition ctor in type.Constructors)
+					yield return ctor;
+			}
+			if (type.HasMethods) {
+				foreach (MethodDefinition method in type.Methods)
+					yield return method;
+			}
+		}
+
+		/// <summary>
+		/// Returns an IEnumerable that allows a single loop (like a foreach) to
+		/// traverse all base classes and interfaces inherited by the type.
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>An IEnumerable to traverse all base classes and interfaces.</returns>
+		public static IEnumerable<TypeDefinition> AllSuperTypes (this TypeReference self)
+		{
+			var types = new List<TypeReference> ();
+			types.Add (self);
+			
+			int i = 0;
+			while (i < types.Count) {
+				TypeDefinition type = types [i++].Resolve ();
+				if (type != null) {
+					yield return type;
+					
+					foreach (TypeReference super in type.Interfaces) {
+						types.AddIfNew (super);
+					}
+					
+					if (type.BaseType != null)
+						types.AddIfNew (type.BaseType);
+				}
+			}
 		}
 
 		/// <summary>
@@ -123,9 +155,31 @@ namespace Gendarme.Framework.Rocks {
 		/// </remarks>
 		public static MethodDefinition GetMethod (this TypeReference self, MethodSignature signature)
 		{
-			foreach (MethodDefinition method in self.AllMethods ()) {
-				if (signature.Matches (method))
-					return method;
+			bool ctors, methods;
+			// method name is optional so we must look in everything
+			if (String.IsNullOrEmpty (signature.Name)) {
+				ctors = true;
+				methods = true;
+			} else {
+				ctors = (signature.Name [0] == '.');
+				methods = !ctors;
+			}
+			
+			TypeDefinition type = self.Resolve ();
+			if (type == null)
+				return null;
+
+			if (ctors && type.HasConstructors) {
+				foreach (MethodDefinition ctor in type.Constructors) {
+					if (signature.Matches (ctor))
+						return ctor;
+				}
+			}
+			if (methods && type.HasMethods) {
+				foreach (MethodDefinition method in type.Methods) {
+					if (signature.Matches (method))
+						return method;
+				}
 			}
 			return null;
 		}
@@ -150,19 +204,23 @@ namespace Gendarme.Framework.Rocks {
 				if (returnType != null && method.ReturnType.ReturnType.FullName != returnType)
 					continue;
 				if (parameters != null) {
-					if (parameters.Length != method.Parameters.Count)
-						continue;
-					bool parameterError = false;
-					for (int i = 0; i < parameters.Length; i++) {
-						if (parameters [i] == null)
-							continue;//ignore parameter
-						if (parameters [i] != method.Parameters [i].ParameterType.GetOriginalType ().FullName) {
-							parameterError = true;
-							break;
+					if (method.HasParameters) {
+						if (parameters.Length != method.Parameters.Count)
+							continue;
+						bool parameterError = false;
+						for (int i = 0; i < parameters.Length; i++) {
+							if (parameters [i] == null)
+								continue;//ignore parameter
+							if (parameters [i] != method.Parameters [i].ParameterType.GetOriginalType ().FullName) {
+								parameterError = true;
+								break;
+							}
 						}
+						if (parameterError)
+							continue; // there could be an overload with the "right" parameters
+					} else if (parameters.Length > 0) {
+						continue;
 					}
-					if (parameterError)
-						continue; // there could be an overload with the "right" parameters
 				}
 				if (customCondition != null && !customCondition (method))
 					continue;
@@ -244,40 +302,47 @@ namespace Gendarme.Framework.Rocks {
 		}
 
 		/// <summary>
-		/// Check if the type implemented a specified interface. Note that it is possible that
-		/// we might now be able to know all implementations since the assembly where 
-		/// the information resides could be unavailable.
+		/// Recursively check if the type implemented a specified interface. Note that it is possible
+		/// that we might now be able to know everything that a type implements since the assembly 
+		/// where the information resides could be unavailable. False is returned in this case.
 		/// </summary>
 		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
 		/// <param name="interfaceName">Full name of the interface</param>
-		/// <returns>True if the type implements the interface, False otherwise.</returns>
+		/// <returns>True if we found that the type implements the interface, False otherwise (either it
+		/// does not implement it, or we could not find where it does).</returns>
 		public static bool Implements (this TypeReference self, string interfaceName)
 		{
 			if (interfaceName == null)
 				throw new ArgumentNullException ("interfaceName");
 
-			bool generic = (interfaceName.IndexOf ('`') >= 0);
-
 			TypeDefinition type = self.Resolve ();
 			if (type == null)
-				return false;		// could not resolve
+				return false;	// not enough information available
 
 			// special case, check if we implement ourselves
 			if (type.IsInterface && (type.FullName == interfaceName))
 				return true;
 
-			// does the type implements it itself
-			foreach (TypeReference iface in type.Interfaces) {
-				string fullname = (generic) ? iface.GetOriginalType ().FullName : iface.FullName;
-				if (fullname == interfaceName)
-					return true;
-			}
-			
-			// if not, then maybe it's parent does
-			TypeReference parent = type.BaseType;
-			if (parent != null)
-				return parent.Implements (interfaceName);
+			return Implements (type, interfaceName, (interfaceName.IndexOf ('`') >= 0));
+		}
 
+		private static bool Implements (TypeDefinition type, string interfaceName, bool generic)
+		{
+			while (type != null) {
+				// does the type implements it itself
+				if (type.HasInterfaces) {
+					foreach (TypeReference iface in type.Interfaces) {
+						string fullname = (generic) ? iface.GetOriginalType ().FullName : iface.FullName;
+						if (fullname == interfaceName)
+							return true;
+						//if not, then maybe one of its parent interfaces does
+						if (Implements (iface.Resolve (), interfaceName, generic))
+							return true;
+					}
+				}
+
+				type = type.BaseType != null ? type.BaseType.Resolve () : null;
+			}
 			return false;
 		}
 
@@ -359,7 +424,7 @@ namespace Gendarme.Framework.Rocks {
 		public static bool IsFlags (this TypeReference self)
 		{
 			TypeDefinition type = self.Resolve ();
-			if (!type.IsEnum)
+			if ((type == null) || !type.IsEnum || !type.HasCustomAttributes)
 				return false;
 
 			return type.HasAttribute ("System.FlagsAttribute");
@@ -384,10 +449,12 @@ namespace Gendarme.Framework.Rocks {
 		/// False otherwise (e.g. compiler or tool generated)</returns>
 		public static bool IsGeneratedCode (this TypeReference self)
 		{
-			// both helpful attributes only exists in 2.0 and more recent frameworks
-			if (self.Module.Assembly.Runtime >= TargetRuntime.NET_2_0) {
-				if (self.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
-					return true;
+			if (self.HasCustomAttributes) {
+				// both helpful attributes only exists in 2.0 and more recent frameworks
+				if (self.Module.Assembly.Runtime >= TargetRuntime.NET_2_0) {
+					if (self.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
+						return true;
+				}
 			}
 
 			// sadly <Module> still shows up for 2.0, so the 1.x logic still applies
@@ -420,6 +487,37 @@ namespace Gendarme.Framework.Rocks {
 			}
 		}
 
+		/// <summary>
+		/// Check if the type refers to a primitive type.
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <returns>True if the type is a primitive type, False otherwise</returns>
+		public static bool IsPrimitive (this TypeReference self)
+		{
+			if ((self == null) || (self.Namespace != "System"))
+				return false;
+
+			switch (self.Name) {
+			case "Byte":
+			case "SByte":
+			case "Boolean":
+			case "Int16":
+			case "UInt16":
+			case "Char":
+			case "Int32":
+			case "UInt32":
+			case "Single":
+			case "Int64":
+			case "UInt64":
+			case "Double":
+			case "IntPtr":
+			case "UIntPtr":
+				return true;
+			default:
+				return false;
+			}
+		}
+
 		/// <summary>
 		/// Check if the type is static (2.0+)
 		/// </summary>
@@ -452,22 +550,5 @@ namespace Gendarme.Framework.Rocks {
 			}
 			return type.IsPublic;
 		}
-
-		/// <summary>
-		/// Resolve a TypeReference into a TypeDefinition.
-		/// </summary>
-		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <returns>A TypeDefinition if resolved, null otherwise.</returns>
-		public static TypeDefinition Resolve (this TypeReference self)
-		{
-			// this can occurs, e.g. generic parameters that needs recursive resolves
-			if (self == null)
-				return null;
-
-			TypeDefinition type = (self as TypeDefinition);
-			if (type == null)
-				type = AssemblyResolver.Resolver.Resolve (self);
-			return type;
-		}
 	}
 }
