@@ -24,20 +24,11 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
-
-using System;
-
-using Mono.Cecil;
-
 namespace Gendarme.Framework {
 
-	public class RunnerEventArgs : EventArgs {
+	public class RunnerEventArgs : HierarchicalEventArgs {
 
 		private IRunner runner;
-		private AssemblyDefinition assembly;
-		private ModuleDefinition module;
-		private TypeDefinition type;
-		private MethodDefinition method;
 
 		public RunnerEventArgs (IRunner runner)
 		{
@@ -47,43 +38,5 @@ namespace Gendarme.Framework {
 		public IRunner Runner {
 			get { return runner; }
 		}
-
-		public AssemblyDefinition CurrentAssembly {
-			get { return assembly; }
-			set {
-				if (value != assembly) {
-					assembly = value;
-					module = null;
-					type = null;
-					method = null;
-				}
-			}
-		}
-
-		public ModuleDefinition CurrentModule {
-			get { return module; }
-			set {
-				if (value != module) {
-					module = value;
-					type = null;
-					method = null;
-				}
-			}
-		}
-
-		public TypeDefinition CurrentType {
-			get { return type; }
-			set {
-				if (value != type) {
-					type = value;
-					method = null;
-				}
-			}
-		}
-
-		public MethodDefinition CurrentMethod {
-			get { return method; }
-			set { method = value; }
-		}
 	}
 }
