@@ -37,10 +37,13 @@ using Gendarme.Framework.Rocks;
 namespace Gendarme.Rules.BadPractice {
 
 	/// <summary>
-	/// This rule check if non-console applications are using <c>Console.WriteLine</c> as
-	/// a mean to help debugging issues. While useful the debugging code should, before its
-	/// release be removed, moved inside methods decorated with <c>[Conditional("DEBUG")]</c> or 
-	/// <c>[Conditional("TRACE")]</c> or changed to use the <c>Debug</c> or <c>Trace</c> types.
+	/// This rule checks for non-console applications which contain calls to <c>Console.WriteLine</c>.
+	/// These are often used as debugging aids but such code should be removed or disabled in 
+	/// the released version. If you don't want to remove it altogether you can place it inside a method
+	/// decorated with <c>[Conditional ("DEBUG")]</c>, use <c>Debug.WriteLine</c>, use
+	/// <c>Trace.WriteLine</c>, or use the preprocessor. But note that TRACE is often enabled
+	/// in release builds so if you do use that you'll probably want to use a config file to remove 
+	/// the default trace listener.
 	/// </summary>
 	/// <example>
 	/// Bad example:
@@ -79,8 +82,8 @@ namespace Gendarme.Rules.BadPractice {
 	/// </example>
 	/// <remarks>This rule is available since Gendarme 2.0</remarks>
 
-	[Problem ("This method include calls to Console.WriteLine inside an assembly not compiled for console application (e.g. /target:exe).")]
-	[Solution ("If this code is used for debugging purpose then either use the Debug or Trace types or disable the code manually (e.g. using a preprocessor).")]
+	[Problem ("This method includes calls to Console.WriteLine inside an assembly not compiled for console application (e.g. /target:exe).")]
+	[Solution ("If this code is used for debugging then either use the Debug or Trace types or disable the code manually (e.g. using the preprocessor).")]
 	[EngineDependency (typeof (OpCodeEngine))]
 	public class DisableDebuggingCodeRule : Rule, IMethodRule {
 
