@@ -33,31 +33,66 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
-	[Problem ("This type contains public instance fields.")]
-	[Solution ("If possible change the public fields into properties.")]
-	public class AvoidPublicInstanceFieldsRule : Rule, ITypeRule {
+	// TODO: It'd be nice if this rule also fired for fields with public access and internal visibility. 
+	// This is a common and serious design error so it seems like it should be reported at
+	// least at low severity.
+
+	/// <summary>
+	/// This rule fires if a type contains externally visible fields. Instead use
+	/// a property which allows you to change the implementation without breaking
+	/// binary compatibility with other assemblies.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public class Foo {
+	///	public int Value;
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public class Foo {
+	///	private int v;
+	///	public int Value {
+	///	get {
+	///		return v;
+	///	}
+	///	set {
+	///		v = value;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <remarks>Prior to Gendarme 2.2 this rule was named AvoidPublicInstanceFieldsRule.</remarks>
+
+	[Problem ("This type contains visible instance fields. A field should be an implementation detail and encapsulated within a property or method.")]
+	[Solution ("Use a property or method instead.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
+	public class AvoidVisibleFieldsRule : Rule, ITypeRule {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			// rule doesn't apply on enums or to compiler/tools-generated code
+			// rule doesn't apply on enums, interfaces, delegates or to compiler/tools-generated code
 			// e.g. CSC compiles anonymous methods as an inner type that expose public fields
-			if (type.IsEnum || type.IsGeneratedCode ())
+			if (type.IsEnum || type.IsInterface || type.IsDelegate () || type.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 			
-			// rule doesn't apply to private or internal nested types
-			if (type.IsNestedPrivate || type.IsNestedAssembly)
+			// rule doesn't apply to type non (externally) visible
+			if (!type.IsVisible ())
 				return RuleResult.DoesNotApply;
 
 			foreach (FieldDefinition fd in type.Fields) {
-				if (!fd.IsPublic || fd.IsSpecialName || fd.IsStatic || fd.HasConstant || fd.IsInitOnly)
+				if (!fd.IsVisible () || fd.IsSpecialName || fd.HasConstant || fd.IsInitOnly)
 					continue;
 
 				if (fd.FieldType.IsArray ()) {
-					string s = String.Format ("Consider converting the public instance field '{0}' into a private field and add a 'Set{1}' method.", 
+					string s = String.Format ("Consider changing the field '{0}' to a private or internal field and add a 'Set{1}' method.",
 						fd.Name, Char.ToUpper (fd.Name [0]) + fd.Name.Substring (1));
 					Runner.Report (fd, Severity.Medium, Confidence.Total, s);
 				} else {
-					string s = String.Format ("Public instance field '{0}' should probably be a property.", fd.Name);
+					string s = String.Format ("Field '{0}' should be private or internal and its value accessed through a property.", fd.Name);
 					Runner.Report (fd, Severity.Medium, Confidence.Total, s);
 				}
 			}
