@@ -40,8 +40,42 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
+	/// <summary>
+	/// This rule will fire if a type contains disposable fields but does not implement
+	/// <c>System.IDisposable</c>.
+	/// </summary>
+	/// <example>
+	/// Bad examples:
+	/// <code>
+	/// class DoesNotImplementIDisposable {
+	///	IDisposable field;
+	/// }
+	/// 
+	/// class AbstractDispose : IDisposable {
+	///	IDisposable field;
+	///	
+	///	// the field should be disposed in the type that declares it
+	///	public abstract void Dispose ();
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// class Dispose : IDisposable {
+	///	IDisposable field;
+	///	
+	///	public void Dispose ()
+	///	{
+	///		field.Dispose ();
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("This type contains disposable field(s) but doesn't implement IDisposable.")]
-	[Solution ("Implement IDisposable and free the disposable field(s) in its Dispose method.")]
+	[Solution ("Implement IDisposable and free the disposable field(s) in the Dispose method.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable")]
 	public class TypesWithDisposableFieldsShouldBeDisposableRule : Rule, ITypeRule {
 
 		private const string AbstractTypeMessage = "Field implement IDisposable. Type should implement a non-abstract Dispose() method";
@@ -50,8 +84,8 @@ namespace Gendarme.Rules.Design {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			// rule doesn't apply to enums, interfaces or structs
-			if (type.IsEnum || type.IsInterface || type.IsValueType || type.IsGeneratedCode ())
+			// rule doesn't apply to enums, interfaces, structs, delegates or generated code
+			if (type.IsEnum || type.IsInterface || type.IsValueType || type.IsDelegate () || type.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 
 			MethodDefinition explicitDisposeMethod = null;
@@ -78,6 +112,9 @@ namespace Gendarme.Rules.Design {
 				TypeDefinition fieldType = field.FieldType.GetOriginalType ().Resolve ();
 				if (fieldType == null)
 					continue;
+				// enums and primitives don't implement IDisposable
+				if (fieldType.IsEnum || fieldType.IsPrimitive ())
+					continue;
 				if (fieldType.Implements ("System.IDisposable")) {
 					if (abstractWarning)
 						Runner.Report (field, Severity.High, Confidence.High, AbstractTypeMessage);
@@ -90,9 +127,6 @@ namespace Gendarme.Rules.Design {
 			if (implicitDisposeMethod != null && implicitDisposeMethod.IsAbstract)
 				Runner.Report (implicitDisposeMethod, Severity.Medium, Confidence.High, AbstractDisposeMessage);
 
-			if (explicitDisposeMethod != null && explicitDisposeMethod.IsAbstract)
-				Runner.Report (explicitDisposeMethod, Severity.Medium, Confidence.High, AbstractDisposeMessage);
-
 			return Runner.CurrentRuleResult;
 		}
 	}
