@@ -34,8 +34,42 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
+	/// <summary>
+	/// This rule will fire if a type contains <c>IntPtr</c>, <c>UIntPtr</c>, or 
+	/// <c>HandleRef</c> fields but does not implement <c>System.IDisposable</c>.
+	/// </summary>
+	/// <example>
+	/// Bad examples:
+	/// <code>
+	/// public class DoesNotImplementIDisposable {
+	///	IntPtr field;
+	/// }
+	/// 
+	/// abstract public class AbstractDispose : IDisposable {
+	///	IntPtr field;
+	///	
+	///	// the field should be disposed in the type that declares it
+	///	public abstract void Dispose ();
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public class Dispose : IDisposable {
+	///	IDisposable field;
+	///	
+	///	public void Dispose ()
+	///	{
+	///		UnmanagedFree (field);
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("This type contains native field(s) but doesn't implement IDisposable.")]
-	[Solution ("Implement IDisposable and free the native field(s) in it's Dispose method.")]
+	[Solution ("Implement IDisposable and free the native field(s) in the Dispose method.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1049:TypesThatOwnNativeResourcesShouldBeDisposable")]
 	public class TypesWithNativeFieldsShouldBeDisposableRule : Rule, ITypeRule {
 
 		private const string AbstractTypeMessage = "Field is native. Type should implement a non-abstract Dispose() method";
@@ -44,8 +78,8 @@ namespace Gendarme.Rules.Design {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			// rule doesn't apply to enums, interfaces or structs
-			if (type.IsEnum || type.IsInterface || type.IsValueType)
+			// rule doesn't apply to enums, interfaces, structs, delegates or generated code
+			if (type.IsEnum || type.IsInterface || type.IsValueType || type.IsDelegate () || type.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 
 			MethodDefinition explicitDisposeMethod = null;
@@ -81,9 +115,6 @@ namespace Gendarme.Rules.Design {
 			if (implicitDisposeMethod != null && implicitDisposeMethod.IsAbstract)
 				Runner.Report (implicitDisposeMethod, Severity.Medium, Confidence.High, AbstractDisposeMessage);
 
-			if (explicitDisposeMethod != null && explicitDisposeMethod.IsAbstract)
-				Runner.Report (explicitDisposeMethod, Severity.Medium, Confidence.High, AbstractDisposeMessage);
-
 			return Runner.CurrentRuleResult;
 		}
 	}
