@@ -1,5 +1,5 @@
 //
-// Gendarme.Rules.Design.UsingCloneWithoutImplementingICloneableRule
+// Gendarme.Rules.Design.ImplementICloneableCorrectlyRule
 //
 // Authors:
 //	Nidhi Rawal <sonu2404@gmail.com>
@@ -36,9 +36,61 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
-	[Problem ("This type provides a Clone() method returning System.Object but does not implement the ICloneable interface.")]
-	[Solution ("Implement the ICloneable interface or change the return type to this type.")]
-	public class UsingCloneWithoutImplementingICloneableRule: Rule, ITypeRule {
+	// TODO: As of .NET 2.0 the Framework Design Guidelines recommend that ICloneable
+	// not be used because it is so poorly specified. See <http://blogs.msdn.com/brada/archive/2003/04/09/49935.aspx>
+	// for example. This rule should probably be replaced with a new rule which fires for
+	// types that implement ICloneable.
+	
+	/// <summary>
+	/// This rule fires if you implement a <c>object Clone()</c> method without 
+	/// implementing the <c>System.ICloneable</c> interface. Either change the
+	/// method so that it returns a better type than System.Object or implement
+	/// ICloneable.
+	/// <list type="bullet"><description>Note: Be sure to document the behavior of your
+	/// Clone method since the framework itself is not very clear, or consistent, between 
+	/// shallow and deep cloning.</description></list>
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public class MyClass {
+	///	public object Clone ()
+	///	{
+	///		MyClass myClass = new MyClass ();
+	///		return myClass;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (ICloneable):
+	/// <code>
+	/// public class MyClass : ICloneable {
+	///	public object Clone ()
+	///	{
+	///		MyClass myClass = new MyClass ();
+	///		return myClass;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (not returning System.Object):
+	/// <code>
+	/// public class MyClass {
+	///	public MyClass Clone ()
+	///	{
+	///		MyClass myClass = new MyClass ();
+	///		return myClass;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <remarks>Prior to Gendarme 2.2 this rule was named UsingCloneWithoutImplementingICloneableRule</remarks>
+
+	[Problem ("This type provides a Clone() method which returns System.Object but does not implement the ICloneable interface.")]
+	[Solution ("Use a better return type or implement the ICloneable interface.")]
+	public class ImplementICloneableCorrectlyRule: Rule, ITypeRule {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
@@ -53,7 +105,7 @@ namespace Gendarme.Rules.Design {
 				if (method.Name != "Clone")
 					continue;
 
-				if (method.Parameters.Count > 0)
+				if (method.HasParameters)
 					continue;
 
 				// that return System.Object, e.g. public object Clone()
