@@ -34,9 +34,54 @@ using Gendarme.Framework;
 
 namespace Gendarme.Rules.Exceptions {
 
-	[Problem ("The method catch a nonspecific exception.")]
-	[Solution ("You can rethrow the original exception, to avoid destroying the stacktrace, or you can handle more specific exceptions.")]
-	public class DontSwallowErrorsCatchingNonspecificExceptionsRule : Rule, IMethodRule {
+	/// <summary>
+	/// This rule will fire if a catch block catches <c>System.Exception</c> or
+	/// <c>System.SystemException</c> but does not rethrow the original
+	/// exception. This is problematic because you don't know what went wrong
+	/// so it's difficult to know that the error was handled correctly. It is better
+	/// to catch a more specific set of exceptions so that you do know what went
+	/// wrong and do know that it is handled correctly.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// try {
+	///	File.Open ("foo.txt", FileMode.Open); 
+	/// }
+	/// catch (Exception) {
+	///	//Ooops  what's failed ??? UnauthorizedException, FileNotFoundException ??? 
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (catch a specific exception):
+	/// <code>
+	/// try {
+	///	File.Open ("foo.txt", FileMode.Open);
+	/// }
+	/// catch (FileNotFoundException exception) {
+	///	//I know that the system can't find the file.
+	/// } 
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (catch all and rethrow):
+	/// <code>
+	/// try {
+	///	File.Open ("foo.txt", FileMode.Open);
+	/// }
+	/// catch {
+	///	Console.WriteLine ("An error has happened.");
+	///	throw;  // You don't swallow the error, because you rethrow the original exception.
+	/// }
+	/// </code>
+	/// </example>
+	/// <remarks>Prior to Gendarme 2.0 this rule was named DontSwallowErrorsCatchingNonspecificExceptionsRule.</remarks>
+
+	[Problem ("This method catches a very general exception without rethrowing it. This is not safe to do in general and may mask problems that the caller should be made aware of.")]
+	[Solution ("Rethrow the original exception (which will preserve the stacktrace of the original error) or catch a more specific exception type.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
+	public class DoNotSwallowErrorsCatchingNonSpecificExceptionsRule : Rule, IMethodRule {
 
 		//Added System.Object because is the code behind the following block:
 		//try {
@@ -58,7 +103,7 @@ namespace Gendarme.Rules.Exceptions {
 
 		private static Instruction ThrowsGeneralException (ExceptionHandler exceptionHandler)
 		{
-			for (Instruction currentInstruction = exceptionHandler.HandlerStart; currentInstruction != exceptionHandler.HandlerEnd; currentInstruction = currentInstruction.Next) {
+			for (Instruction currentInstruction = exceptionHandler.HandlerStart; currentInstruction != exceptionHandler.HandlerEnd && currentInstruction != null; currentInstruction = currentInstruction.Next) {
 				if (currentInstruction.OpCode.Code == Code.Rethrow)
 					return null;
 			}
@@ -71,14 +116,18 @@ namespace Gendarme.Rules.Exceptions {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// and if the method has, at least one, exception handler(s)
 			ExceptionHandlerCollection exceptionHandlerCollection = method.Body.ExceptionHandlers;
+			if (exceptionHandlerCollection.Count == 0)
+				return RuleResult.DoesNotApply;
+
 			foreach (ExceptionHandler exceptionHandler in exceptionHandlerCollection) {
 				if (exceptionHandler.Type == ExceptionHandlerType.Catch) {
 					string catchTypeName = exceptionHandler.CatchType.FullName;
 					if (IsForbiddenTypeInCatches (catchTypeName)) {
 						Instruction throw_instruction = ThrowsGeneralException (exceptionHandler);
 						if (throw_instruction != null) {
-							Runner.Report (method, throw_instruction, Severity.Medium, Confidence.High, String.Empty);
+							Runner.Report (method, throw_instruction, Severity.Medium, Confidence.High);
 						}
 					}
 				}
