@@ -33,8 +33,38 @@ using Gendarme.Framework;
 
 namespace Gendarme.Rules.Interoperability {
 
-	[Problem ("Marshaling information for string types is incomplete and what is required may be different from what you expected the default to be.")]
-	[Solution ("Add [DllImport CharSet=] on the method or [MarshalAs] on the parameter(s)")]
+	/// <summary>
+	/// This rule will fire if a P/Invoke method has System.String or System.Text.StringBuilder
+	/// arguments, and the DllImportAttribute does not specify the <code>CharSet</code>, 
+	/// and the string arguments are not decorated with <code>[MarshalAs]</code>.
+	///
+	/// This is important because the defaults are different on the various platforms.
+	/// On Mono the default is to always use utf-8. On .NET the default is to use the ANSI
+	/// CharSet which is the native encoding and will typically be some variant of ASCII or
+	/// something like Shift-JIS. On Compact .NET the default is utf-16.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// [DllImport ("coredll")]
+	/// static extern int SHCreateShortcut (StringBuilder szShortcut, StringBuilder szTarget);
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good examples:
+	/// <code>
+	/// [DllImport ("coredll", CharSet = CharSet.Auto)]
+	/// static extern int SHCreateShortcut (StringBuilder szShortcut, StringBuilder szTarget);
+	/// 
+	/// [DllImport ("coredll")]
+	/// static extern int SHCreateShortcut ([MarshalAs (UnmanagedType.LPTStr)] StringBuilder szShortcut, 
+	///	[MarshalAs (UnmanagedType.LPTStr)] StringBuilder szTarget);
+	/// </code>
+	/// </example>
+
+	[Problem ("Marshaling information for string types is missing and what is required may be different from what you expected the default to be.")]
+	[Solution ("Add [DllImport CharSet=] to the method or [MarshalAs] on the parameter(s)")]
+	[FxCopCompatibility ("Microsoft.Globalization", "CA2101:SpecifyMarshalingForPInvokeStringArguments")]
 	public class MarshalStringsInPInvokeDeclarationsRule : Rule, IMethodRule {
 
 		private static bool IsStringOrSBuilder (TypeReference reference)
@@ -54,7 +84,7 @@ namespace Gendarme.Rules.Interoperability {
 			if (!method.IsPInvokeImpl)
 				return RuleResult.DoesNotApply;
 
-			if (!method.PInvokeInfo.IsCharSetNotSpec)
+			if (!method.PInvokeInfo.IsCharSetNotSpec || !method.HasParameters)
 				return RuleResult.Success;
 
 			foreach (ParameterDefinition parameter in method.Parameters) {
