@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.ComponentModel;
 
 using Mono.Cecil;
 
@@ -36,29 +37,38 @@ using Gendarme.Framework.Rocks;
 namespace Gendarme.Rules.Maintainability {
 
 	/// <summary>
-	/// This rule checks every type to see how deep is it's inheritance tree inside
-	/// the analyzed assembly set. By default (configurable) the rule will warn if the
-	/// depth is greater than four levels. Optionally (configurable) it can include any
-	/// resolvable assembly (out of the analyzed assembly set) in the check.
+	/// This rule will fire if a type has (by default) more than four base classes defined
+	/// within the assembly set being analyzed. Optionally it will also count base 
+	/// classes defined outside the assembly set being analyzed.
 	/// </summary>
 	/// <remarks>This rule is available since Gendarme 2.0</remarks>
 
-	[Problem ("This type inheritance tree is more than four levels deep.")]
+	[Problem ("This type has more than four base classes.")]
 	[Solution ("Refactor your class hierarchy to reduce its depth. Consider using extension methods to extend existing types.")]
 	[FxCopCompatibility ("Microsoft.Maintainability", "CA1501:AvoidExcessiveInheritance")]
 	public class AvoidDeepInheritanceTreeRule : Rule, ITypeRule {
 
+		/// <summary>Classes with more base classes than this will result in a defect.</summary>
+		/// <remarks>Defaults to 4.</remarks>
+		[DefaultValue (DefaultMaximumDepth)]
+		[Description ("Classes with more base classes than this will result in a defect.")]
 		public int MaximumDepth {
 			get { return maximumDepth; }
 			set { maximumDepth = value; }
 		}
-		private int maximumDepth = 4;
+		private const int DefaultMaximumDepth = 4;
+		private int maximumDepth = DefaultMaximumDepth;
 
+		/// <summary>If true the rule will count base classes defined outside the assemblies being analyzed.</summary>
+		/// <remarks>Defaults to false.</remarks>
+		[DefaultValue (DefaultCountExternalDepth)]
+		[Description ("If true the rule will count base classes defined outside the assemblies being analyzed.")]
 		public bool CountExternalDepth {
 			get { return countExternalDepth; }
 			set { countExternalDepth = value; }
 		}
-		private bool countExternalDepth = false;
+		private const bool DefaultCountExternalDepth = false;
+		private bool countExternalDepth = DefaultCountExternalDepth;
 
 
 		private Severity GetSeverity (int depth)
