@@ -35,11 +35,60 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Naming {
 
-	[Problem ("This method overrides (or implement) an existing method but does not use the same parameter names as the original.")]
+	/// <summary>
+	/// This rule warns if an overriden method's parameter names does not match those of the 
+	/// base class or those of the implemented interface. This can be confusing because it may
+	/// not always be clear that it is an override or implementation of an interface method. It
+	/// also makes it more difficult to use the method with languages that support named
+	/// parameters (like C# 4.0).
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public class Base {
+	///	public abstract void Write (string text);
+	/// }
+	/// 
+	/// public class SubType : Base {
+	///	public override void Write (string output)
+	///	{
+	///		//...
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public class Base {
+	///	public abstract void Write (string text);
+	/// }
+	/// 
+	/// class SubType : Base {
+	///	public override void Write (string text)
+	///	{
+	///		//...
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
+	[Problem ("This method overrides (or implements) an existing method but does not use the same parameter names as the original.")]
 	[Solution ("Keep parameter names consistent when overriding a class or implementing an interface.")]
+	[FxCopCompatibility ("Microsoft.Naming", "CA1725:ParameterNamesShouldMatchBaseDeclaration")]
 	public class ParameterNamesShouldMatchOverriddenMethodRule : Rule, IMethodRule {
 
-		private static bool SignatureMatches (MethodDefinition method, MethodDefinition baseMethod, bool explicitInterfaceCheck)
+		public override void Initialize (IRunner runner)
+		{
+			base.Initialize (runner);
+
+			//check if this is a Boo assembly using macros
+			Runner.AnalyzeAssembly += delegate (object o, RunnerEventArgs e) {
+				IsBooAssemblyUsingMacro = (e.CurrentAssembly.MainModule.TypeReferences.ContainsType (BooMacroStatement));
+			};
+		}
+
+		private static bool SignatureMatches (MethodReference method, MethodReference baseMethod, bool explicitInterfaceCheck)
 		{
 			if (method.Name != baseMethod.Name) {
 				if (!explicitInterfaceCheck)
@@ -47,7 +96,7 @@ namespace Gendarme.Rules.Naming {
 				if (method.Name != baseMethod.DeclaringType.FullName + "." + baseMethod.Name)
 					return false;
 			}
-			if (method.ReturnType.ToString () != baseMethod.ReturnType.ToString ())
+			if (method.ReturnType.ReturnType.FullName != baseMethod.ReturnType.ReturnType.FullName)
 				return false;
 			if (method.Parameters.Count != baseMethod.Parameters.Count)
 				return false;
@@ -97,7 +146,7 @@ namespace Gendarme.Rules.Naming {
 
 		public RuleResult CheckMethod (MethodDefinition method)
 		{
-			if (!method.IsVirtual || method.IsGeneratedCode ())
+			if (!method.IsVirtual || !method.HasParameters || method.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 
 			MethodDefinition baseMethod = null;
@@ -108,6 +157,10 @@ namespace Gendarme.Rules.Naming {
 			if (baseMethod == null)
 				return RuleResult.Success;
 
+			//do not trigger false positives on Boo macros
+			if (IsBooAssemblyUsingMacro && IsBooMacroParameter (baseMethod.Parameters [0]))
+				return RuleResult.Success;
+
 			for (int i = 0; i < method.Parameters.Count; i++) {
 				if (method.Parameters [i].Name != baseMethod.Parameters [i].Name) {
 					string s = string.Format (CultureInfo.InstalledUICulture,
@@ -118,5 +171,22 @@ namespace Gendarme.Rules.Naming {
 			}
 			return Runner.CurrentRuleResult;
 		}
+
+		private const string BooMacroStatement = "Boo.Lang.Compiler.Ast.MacroStatement";
+
+		private bool IsBooAssemblyUsingMacro {
+			get {
+				return isBooAssemblyUsingMacro;
+			}
+			set {
+				isBooAssemblyUsingMacro = value;
+			}
+		}
+		private bool isBooAssemblyUsingMacro;
+
+		private static bool IsBooMacroParameter (ParameterReference p)
+		{
+			return p.Name == "macro" && p.ParameterType.FullName == BooMacroStatement;
+		}
 	}
 }
