@@ -29,13 +29,10 @@
 using System;
 using System.Collections;
 using System.Collections.Generic;
-using System.Linq;
 using System.Reflection;
 using System.Runtime.Serialization;
 using System.Runtime.InteropServices;
 
-using Gendarme.Framework;
-using Gendarme.Framework.Rocks;
 using Gendarme.Rules.Performance;
 using Mono.Cecil;
 
@@ -44,7 +41,26 @@ using Test.Rules.Fixtures;
 
 namespace Test.Rules.Performance {
 
-#pragma warning disable 169
+#pragma warning disable 169, 414
+
+	// from Mono.Rocks
+	// note: error CS1109: Extension methods must be defined in a top level static class; StaticType is a nested class
+	public static class StaticType {
+		public static IEnumerable<TSource> Repeat<TSource> (this IEnumerable<TSource> self, int number)
+		{
+			//				Check.Self (self);
+
+			return CreateRepeatIterator (self, number);
+		}
+
+		private static IEnumerable<TSource> CreateRepeatIterator<TSource> (IEnumerable<TSource> self, int number)
+		{
+			for (int i = 0; i < number; i++) {
+				foreach (var element in self)
+					yield return element;
+			}
+		}
+	}
 
 	[TestFixture]
 	public class AvoidUncalledPrivateCodeTest : MethodRuleTestFixture<AvoidUncalledPrivateCodeRule> {
@@ -628,7 +644,8 @@ namespace Test.Rules.Performance {
 		}
 
 		[Test]
-		[Ignore ("Fails due to mono bug under investigation")]
+		[Ignore ("Mono bug #320901")]
+		// https://bugzilla.novell.com/show_bug.cgi?id=320901
 		public void Generics ()
 		{
 			AssertRuleSuccess<Anculus> ();
@@ -743,5 +760,66 @@ namespace Test.Rules.Performance {
 			AssertRuleSuccess<EventCases> ("add_Bar");
 			AssertRuleSuccess<EventCases> ("remove_Bar");
 		}
+
+		[Test]
+		[Ignore ("Mono bug #320901")]
+		// https://bugzilla.novell.com/show_bug.cgi?id=320901
+		public void MonoRocks ()
+		{
+			AssertRuleSuccess (typeof (StaticType), "CreateRepeatIterator");
+		}
+
+		// Test cases devired from bug #458178
+
+		void MultidimArray ()
+		{
+			const int LengthA = 2;
+			const int LengthB = 2;
+
+			bool [,] array = new bool [LengthA, LengthB];
+
+			for (int i = 0; i < LengthA; i++) {
+				for (int j = 0; j < LengthB; j++) {
+					array [i, j] = true;
+				}
+			}
+		}
+
+		void JaggedArray ()
+		{
+			const int LengthA = 2;
+			const int LengthB = 2;
+
+			bool [][] array = new bool [LengthA][];
+
+			for (int i = 0; i < LengthA; i++) {
+				for (int j = 0; j < LengthB; j++) {
+					array [i][j] = true;
+				}
+			}
+		}
+
+		void NullableArray ()
+		{
+			const int LengthA = 2;
+			const int LengthB = 2;
+
+			bool? [,] array = new bool? [LengthA, LengthB];
+
+			for (int i = 0; i < LengthA; i++) {
+				for (int j = 0; j < LengthB; j++) {
+					array [i, j] = true;
+				}
+			}
+		}
+
+		[Test]
+		// https://bugzilla.novell.com/show_bug.cgi?id=458178
+		public void Arrays ()
+		{
+			AssertRuleFailure<AvoidUncalledPrivateCodeTest> ("MultidimArray", 1);
+			AssertRuleFailure<AvoidUncalledPrivateCodeTest> ("JaggedArray", 1);
+			AssertRuleFailure<AvoidUncalledPrivateCodeTest> ("NullableArray", 1);
+		}
 	}
 }
