@@ -32,10 +32,12 @@ using System.Reflection;
 using System.Runtime.InteropServices;
 
 using Mono.Cecil;
+using Mono.Cecil.Cil;
 using Gendarme.Framework;
 using Gendarme.Rules.Performance;
 
 using NUnit.Framework;
+using Test.Rules.Definitions;
 using Test.Rules.Fixtures;
 using Test.Rules.Helpers;
 
@@ -60,11 +62,6 @@ namespace Test.Rules.Performance {
 	[TestFixture]
 	public class AvoidUnusedParametersTest : MethodRuleTestFixture<AvoidUnusedParametersRule> {
 		
-		private IMethodRule rule;
-		private AssemblyDefinition assembly;
-		private MethodDefinition method;
-		private TestRunner runner;
-
 		public void PrintBannerUsingParameter (Version version) 
 		{
 			Console.WriteLine ("Welcome to the foo program {0}", version);
@@ -93,6 +90,12 @@ namespace Test.Rules.Performance {
 			Console.WriteLine ("Method with unused parameters");
 		}
 
+		[Test]
+		public void MethodWithUnusedParametersTest ()
+		{
+			AssertRuleFailure<AvoidUnusedParametersTest> ("MethodWithUnusedParameters", 2);
+		}
+
 		public void MethodWith5UsedParameters (int x, IEnumerable enumerable, string foo, char c, float f) 
 		{
 			Console.WriteLine (f);
@@ -102,16 +105,34 @@ namespace Test.Rules.Performance {
 			Console.WriteLine (x);
 		}
 
+		[Test]
+		public void MethodWith5UsedParametersTest ()
+		{
+			AssertRuleSuccess<AvoidUnusedParametersTest> ("MethodWith5UsedParameters");
+		}
+
 		public static void StaticMethodWithUnusedParameters (int x, string foo) 
 		{
 		}
 
+		[Test]
+		public void StaticMethodWithUnusedParametersTest ()
+		{
+			AssertRuleFailure<AvoidUnusedParametersTest> ("StaticMethodWithUnusedParameters", 2);
+		}
+
 		public static void StaticMethodWithUsedParameters (int x, string foo) 
 		{
 			Console.WriteLine (x);
 			Console.WriteLine (foo);
 		}
 
+		[Test]
+		public void StaticMethodWithUsedParametersTest ()
+		{
+			AssertRuleSuccess<AvoidUnusedParametersTest> ("StaticMethodWithUsedParameters");
+		}
+
 		public static void StaticMethodWith5UsedParameters (int x, string foo, IEnumerable enumerable, char c, float f) 
 		{
 			Console.WriteLine (f);
@@ -121,13 +142,22 @@ namespace Test.Rules.Performance {
 			Console.WriteLine (x);
 		}
 
+		[Test]
+		public void StaticMethodWith5UsedParametersTest ()
+		{
+			AssertRuleSuccess<AvoidUnusedParametersTest> ("StaticMethodWith5UsedParameters");
+		}
+
 		public static void StaticMethodWith5UnusedParameters (int x, string foo, IEnumerable enumerable, char c, float f)
 		{
 		}
-		
-		[DllImport ("libc.so")]
-		private static extern double cos (double x);
 
+		[Test]
+		public void StaticMethodWith5UnusedParametersTest ()
+		{
+			AssertRuleFailure<AvoidUnusedParametersTest> ("StaticMethodWith5UnusedParameters", 5);
+		}
+		
 		public delegate void SimpleCallback (int x);
 		public void SimpleCallbackImpl (int x) 
 		{
@@ -137,6 +167,23 @@ namespace Test.Rules.Performance {
 		{
 		}
 
+		[Test]
+		public void DelegateMethodTest ()
+		{
+			SimpleCallback callback = new SimpleCallback (SimpleCallbackImpl);
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("SimpleCallbackImpl");
+		}
+
+		[Test]
+		public void DelegateMethodTestWithMultipleDelegates ()
+		{
+			SimpleCallback callback = new SimpleCallback (SimpleCallbackImpl);
+			SimpleCallback callback2 = new SimpleCallback (SimpleCallbackImpl2);
+
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("SimpleCallbackImpl");
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("SimpleCallbackImpl2");
+		}
+
 		public void AnonymousMethodWithUnusedParameters ()
 		{
 			SimpleCallback callback = delegate (int x) {
@@ -144,16 +191,51 @@ namespace Test.Rules.Performance {
 			};
 		}
 
+		[Test]
+		public void AnonymousMethodTest ()
+		{
+			MethodDefinition method = null;
+			// compiler generated code is compiler dependant, check for [g]mcs (inner type)
+			TypeDefinition type = DefinitionLoader.GetTypeDefinition (typeof (AvoidUnusedParametersTest).Assembly, "AvoidUnusedParametersTest/<>c__CompilerGenerated0");
+			if (type != null)
+				method = DefinitionLoader .GetMethodDefinition (type, "<AnonymousMethodWithUnusedParameters>c__2", null);
+			// otherwise try for csc (inside same class)
+			if (method == null) {
+				type = DefinitionLoader.GetTypeDefinition<AvoidUnusedParametersTest> ();
+				foreach (MethodDefinition md in type.Methods) {
+					if (md.Name.StartsWith ("<AnonymousMethodWithUnusedParameters>")) {
+						method = md;
+						break;
+					}
+				}
+			}
+			Assert.IsNotNull (method, "method not found!");
+			AssertRuleDoesNotApply (method);
+		}
+
 		public delegate void SimpleEventHandler (int x);
 		public event SimpleEventHandler SimpleEvent;
 		public void OnSimpleEvent (int x) 
 		{
 		}
 
+		[Test]
+		public void EventTest ()
+		{
+			SimpleEvent += new SimpleEventHandler (OnSimpleEvent);
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("OnSimpleEvent");
+		} 
+
 		public void EmptyMethod (int x) 
 		{
 		}
 
+		[Test]
+		public void EmptyMethodTest ()
+		{
+			AssertRuleFailure<AvoidUnusedParametersTest> ("EmptyMethod", 1);
+		}
+
 		public static bool operator == (AvoidUnusedParametersTest t1, AvoidUnusedParametersTest t2)
 		{
 			return t1.Equals (t2);
@@ -190,6 +272,34 @@ namespace Test.Rules.Performance {
 			}
 		}
 
+		[Test]
+		public void OperatorsClass ()
+		{
+			AssertRuleSuccess<AvoidUnusedParametersTest> ("op_Equality");
+			AssertRuleSuccess<AvoidUnusedParametersTest> ("op_Inequality");
+		}
+
+		[Test]
+		public void OperatorsStructureOk ()
+		{
+			AssertRuleSuccess<AvoidUnusedParametersTest.StructureOk> ("op_Equality");
+			AssertRuleSuccess<AvoidUnusedParametersTest.StructureOk> ("op_Inequality");
+		}
+
+		[Test]
+		public void OperatorsStructureBad ()
+		{
+			AssertRuleFailure<AvoidUnusedParametersTest.StructureBad> ("op_Equality", 2);
+			AssertRuleFailure<AvoidUnusedParametersTest.StructureBad> ("op_Inequality", 2);
+		}
+
+		[Test]
+		public void OperatorsCecil ()
+		{
+			AssertRuleSuccess<OpCode> ("op_Equality");
+			AssertRuleSuccess<OpCode> ("op_Inequality");
+		}
+
 		public void ButtonClick_EvenArgsUnused (object o, EventArgs e)
 		{
 			if (o == null)
@@ -201,223 +311,39 @@ namespace Test.Rules.Performance {
 			Console.WriteLine ("uho");
 		}
 
-		[TestFixtureSetUp]
-		public void FixtureSetUp () 
-		{
-			string unit = Assembly.GetExecutingAssembly ().Location;
-			assembly = AssemblyFactory.GetAssembly (unit);
-			rule = new AvoidUnusedParametersRule ();
-			runner = new TestRunner (rule);
-		}
-
- 
-
 		[Test]
-		public void MethodWithUnusedParametersTest () 
+		public void EventArgs ()
 		{
-			method = GetMethodForTest ("MethodWithUnusedParameters");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method));
-			Assert.AreEqual (2, runner.Defects.Count);
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("ButtonClick_EvenArgsUnused");
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("ButtonClick_NoParameterUnused");
 		}
-		
+
 		[Test]
 		public void AbstractMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AbstractClass", "AbstractMethod");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
+			AssertRuleDoesNotApply<AbstractClass> ("AbstractMethod");
 		}
 
 		[Test]
 		public void VirtualMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.VirtualClass", "VirtualMethod");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
+			AssertRuleDoesNotApply<VirtualClass> ("VirtualMethod");
 		}
 
 		[Test]
 		public void OverrideMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.OverrideClass", "VirtualMethod");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
+			AssertRuleDoesNotApply<OverrideClass> ("VirtualMethod");
 		}
-		
+
+		// using our own extern helps more in coverage than SimpleMethods.External would
+		[DllImport ("libc.so")]
+		private static extern double cos (double x);
+
 		[Test]
 		public void ExternMethodTest () 
 		{
-			method = GetMethodForTest ("cos");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void DelegateMethodTest () 
-		{
-			SimpleCallback callback = new SimpleCallback (SimpleCallbackImpl);
-			method = GetMethodForTest ("SimpleCallbackImpl");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void DelegateMethodTestWithMultipleDelegates () 
-		{
-			SimpleCallback callback = new SimpleCallback (SimpleCallbackImpl);
-			SimpleCallback callback2 = new SimpleCallback (SimpleCallbackImpl2);
-
-			method = GetMethodForTest ("SimpleCallbackImpl");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-
-			method = GetMethodForTest ("SimpleCallbackImpl2");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void AnonymousMethodTest ()
-		{
-			// compiler generated code is compiler dependant, check for [g]mcs (inner type)
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest/<>c__CompilerGenerated0", "<AnonymousMethodWithUnusedParameters>c__2");
-			// otherwise try for csc (inside same class)
-			if (method == null) {
-				TypeDefinition type = assembly.MainModule.Types ["Test.Rules.Performance.AvoidUnusedParametersTest"];
-				foreach (MethodDefinition md in type.Methods) {
-					if (md.Name.StartsWith ("<AnonymousMethodWithUnusedParameters>")) {
-						method = md;
-						break;
-					}
-				}
-			}
-			Assert.IsNotNull (method, "method not found!");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void EventTest () 
-		{
-			SimpleEvent += new SimpleEventHandler (OnSimpleEvent);
-			method = GetMethodForTest ("OnSimpleEvent");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method));
-		} 
-
-		[Test]
-		public void MethodWith5UsedParametersTest () 
-		{
-			method = GetMethodForTest ("MethodWith5UsedParameters");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void StaticMethodWithUnusedParametersTest () 
-		{
-			method = GetMethodForTest ("StaticMethodWithUnusedParameters");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method));
-			Assert.AreEqual (2, runner.Defects.Count);
-		}
-
-		[Test]
-		public void StaticMethodWithUsedParametersTest () 
-		{
-			method = GetMethodForTest ("StaticMethodWithUsedParameters");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method));
-		}
-
-		[Test]
-		public void StaticMethodWith5UsedParametersTest () 
-		{
-			method = GetMethodForTest ("StaticMethodWith5UsedParameters");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method));
-		}
-		
-		[Test]
-		public void StaticMethodWith5UnusedParametersTest () 
-		{
-			method = GetMethodForTest ("StaticMethodWith5UnusedParameters");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method));
-			Assert.AreEqual (5, runner.Defects.Count);
-		}
-
-		[Test]
-		public void EmptyMethodTest ()
-		{
-			method = GetMethodForTest ("EmptyMethod");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method));
-			Assert.AreEqual (1, runner.Defects.Count);
-		}
-
-		[Test]
-		public void OperatorsClass ()
-		{
-			method = GetMethodForTest ("op_Equality");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method), "op_Equality");
-
-			method = GetMethodForTest ("op_Inequality");
-			Assert.AreEqual (RuleResult.Success, rule.CheckMethod (method), "op_Inequality");
-		}
-
-		[Test]
-		public void OperatorsStructureOk ()
-		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest/StructureOk", "op_Equality");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method), "op_Equality");
-
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest/StructureOk", "op_Inequality");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method), "op_Inequality");
-		}
-
-		[Test]
-		public void OperatorsStructureBad ()
-		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest/StructureBad", "op_Equality");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method), "op_Equality");
-			Assert.AreEqual (2, runner.Defects.Count);
-
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest/StructureBad", "op_Inequality");
-			Assert.AreEqual (RuleResult.Failure, runner.CheckMethod (method), "op_Inequality");
-			Assert.AreEqual (2, runner.Defects.Count);
-		}
-
-		[Test]
-		public void OperatorsCecil ()
-		{
-			string cecil = typeof (Mono.Cecil.Cil.OpCode).Assembly.Location;
-			AssemblyDefinition ad = AssemblyFactory.GetAssembly (cecil);
-
-			method = GetMethodFromAssembly (ad, "Mono.Cecil.Cil.OpCode", "op_Equality");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method), "op_Equality");
-
-			method = GetMethodFromAssembly (ad, "Mono.Cecil.Cil.OpCode", "op_Inequality");
-			Assert.AreEqual (RuleResult.Success, runner.CheckMethod (method), "op_Inequality");
-		}
-
-		[Test]
-		public void EventArgs ()
-		{
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest", "ButtonClick_EvenArgsUnused");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method), "ButtonClick_EvenArgsUnused/Result");
-			Assert.AreEqual (0, runner.Defects.Count, "ButtonClick_EvenArgsUnused/Count");
-
-			method = GetMethodForTestFrom ("Test.Rules.Performance.AvoidUnusedParametersTest", "ButtonClick_NoParameterUnused");
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckMethod (method), "ButtonClick_NoParameterUnused/Result");
-			Assert.AreEqual (0, runner.Defects.Count, "ButtonClick_NoParameterUnused/Count");
-		}
-
-		private MethodDefinition GetMethodForTest (string methodName) 
-		{
-			return GetMethodFromAssembly (assembly, "Test.Rules.Performance.AvoidUnusedParametersTest", methodName);
-		}
-
-		private MethodDefinition GetMethodForTestFrom (string fullTypeName, string methodName) 
-		{
-			return GetMethodFromAssembly (assembly, fullTypeName, methodName);
-		}
-
-		private MethodDefinition GetMethodFromAssembly (AssemblyDefinition ad, string fullTypeName, string methodName)
-		{
-			TypeDefinition type = ad.MainModule.Types[fullTypeName];
-			if (type != null) {
-				foreach (MethodDefinition method in type.Methods) {
-					if (method.Name == methodName)
-						return method;
-				}
-			}
-			return null;
+			AssertRuleDoesNotApply<AvoidUnusedParametersTest> ("cos");
 		}
 	}
 }
