@@ -34,14 +34,52 @@ using System.Diagnostics;
 using Mono.Cecil;
 using Mono.Cecil.Cil;
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
+using Gendarme.Framework.Helpers;
 using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Portability {
 
-	[Problem ("The method use some features that requires 'root' priviledge under Unix.")]
-	[Solution ("Make sure your code can work without requiring users to have 'root' priviledge.")]
+	/// <summary>
+	/// This rule fires if a feature is used which is, by default, restricted under Unix.
+	/// <list type="bullet">
+	/// <item>
+	/// <description><c>System.Net.NetworkInformation.Ping</c>: This type can only be used
+	/// by root on Unix systems. As an alternative you can execute the ping command and 
+	/// parse its result.</description>
+	/// </item>
+	/// <item>
+	/// <description><c>System.Diagnostics.Process</c>: The PriorityClass property can only
+	/// be set to <c>Normal</c> by non-root users. To avoid this problem you can do a 
+	/// platform check before assigning a priority.</description>
+	/// </item>
+	/// </list>
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// process.PriorityClass = ProcessPriorityClass.AboveNormal;
+	/// process.Start ();
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// if (Environment.OSVersion.Platform != PlatformID.Unix) {
+	///	process.PriorityClass = ProcessPriorityClass.AboveNormal;
+	/// }
+	/// process.Start ();
+	/// </code>
+	/// </example>
+
+	[Problem ("The method uses some features which require 'root' privilege under Unix.")]
+	[Solution ("Make sure your code can work without requiring users to have 'root' privilege.")]
+	[EngineDependency (typeof (OpCodeEngine))]
 	public class FeatureRequiresRootPrivilegeOnUnixRule : Rule, IMethodRule {
 
+		// includes Call, Callvirt and Newobj
+		private static OpCodeBitmask CallsNew = new OpCodeBitmask (0x8000000000, 0x4400000000000, 0x0, 0x0);
+
 		// localizable
 		private const string ProcessMessage = "Setting Process.PriorityClass to something else than ProcessPriorityClass.Normal requires root privileges.";
 		private const string PingMessage = "Usage of System.Net.NetworkInformation.Ping requires root privileges.";
@@ -103,6 +141,10 @@ namespace Gendarme.Rules.Portability {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// avoid looping if we're sure there's no call in the method
+			if (!CallsNew.Intersect (OpCodeEngine.GetBitmask (method)))
+				return RuleResult.DoesNotApply;
+
 			foreach (Instruction ins in method.Body.Instructions) {
 
 				// check for calls (or newobj)
