@@ -32,11 +32,33 @@ using Mono.Cecil;
 using Mono.Cecil.Cil;
 
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
+using Gendarme.Framework.Helpers;
 
 namespace Gendarme.Rules.Portability {
 
-	[Problem ("The method use some literal values for new lines (e.g. \\r\\n) which aren't portable across operating systems.")]
-	[Solution ("Replace literals with Environment.NewLine.")]
+	/// <summary>
+	/// This rule warns if methods, including properties, are using the literal 
+	/// <c>\r</c> and/or <c>\n</c> for new lines. This isn't portable across operating systems.
+	/// To ensure correct cross-platform functionality they should be replaced by 
+	/// <c>System.Environment.NewLine</c>.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// Console.WriteLine ("Hello,\nYou should be using Gendarme!");
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// Console.WriteLine ("Hello,{0}You must be using Gendarme!", Environment.NewLine);
+	/// </code>
+	/// </example>
+
+	[Problem ("The method uses literals for new lines (e.g. \\r\\n) which isn't portable across operating systems.")]
+	[Solution ("Replace the literals with Environment.NewLine.")]
+	[EngineDependency (typeof (OpCodeEngine))]
 	public class NewLineLiteralRule : Rule, IMethodRule {
 
 		private static char[] InvalidChar = { '\r', '\n' };
@@ -47,26 +69,28 @@ namespace Gendarme.Rules.Portability {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// is there any Ldstr instructions in this method
+			if (!OpCodeEngine.GetBitmask (method).Get (Code.Ldstr))
+				return RuleResult.DoesNotApply;
+
 			// rule applies
 
 			foreach (Instruction ins in method.Body.Instructions) {
-				switch (ins.OpCode.Code) {
-				case Code.Ldstr:
-					// check the string being referenced by the instruction
-					string s = (ins.Operand as string);
-					if (s == null)
-						continue;
+				// look for a string load
+				if (ins.OpCode.Code != Code.Ldstr)
+					continue;
 
-					if (s.IndexOfAny (InvalidChar) >= 0) {
-						// make the invalid char visible on output
-						s = s.Replace ("\n", "\\n");
-						s = s.Replace ("\r", "\\r");
-						s = String.Format ("Found string: \"{0}\"", s);
-						Runner.Report (method, ins, Severity.Low, Confidence.High, s);
-					}
-					break;
-				default:
-					break;
+				// check the string being referenced by the instruction
+				string s = (ins.Operand as string);
+				if (String.IsNullOrEmpty (s))
+					continue;
+
+				if (s.IndexOfAny (InvalidChar) >= 0) {
+					// make the invalid char visible on output
+					s = s.Replace ("\n", "\\n");
+					s = s.Replace ("\r", "\\r");
+					s = String.Format ("Found string: \"{0}\"", s);
+					Runner.Report (method, ins, Severity.Low, Confidence.High, s);
 				}
 			}
 
