@@ -35,9 +35,9 @@ using Gendarme.Framework.Rocks;
 namespace Gendarme.Rules.Serialization {
 
 	/// <summary>
-	/// This rule checks for methods using the special serialization attributes, 
+	/// This rule checks for methods which use the serialization attributes:
 	/// <c>[OnSerializing, OnDeserializing, OnSerialized, OnDeserialized]</c>. You must
-	/// ensure that the methods have the correct signature. They should be <c>private</c>, 
+	/// ensure that these methods have the correct signature. They should be <c>private</c>, 
 	/// return <c>void</c> and have a single parameter of type <c>StreamingContext</c>. 
 	/// Failure to have the right signature can, in some circumstances, make your assembly
 	/// unusable at runtime.
@@ -69,7 +69,7 @@ namespace Gendarme.Rules.Serialization {
 	/// <remarks>This rule is available since Gendarme 2.0</remarks>
 
 	[Problem ("The method has the wrong signature, it should return System.Void and have a single parameter of type 'System.Runtime.Serialization.StreamingContext' and be private.")]
-	[Solution ("Fix method signature to match the runtime requirements.")]
+	[Solution ("Fix the method signature to match the runtime requirements.")]
 	[FxCopCompatibility ("Microsoft.Usage", "CA2238:ImplementSerializationMethodsCorrectly")]
 	public class UseCorrectSignatureForSerializationMethodsRule : Rule, IMethodRule {
 
