@@ -35,17 +35,85 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Smells {
 
+	/// <summary>
+	/// This rule looks for code duplicated in sibling subclasses.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public class BaseClassWithCodeDuplicated {
+	///	protected IList list;
+	/// }
+	///
+	/// public class OverriderClassWithCodeDuplicated : BaseClassWithCodeDuplicated {
+	/// 	public void CodeDuplicated ()
+	/// 	{
+	///		foreach (int i in list) {
+	///			Console.WriteLine (i);
+	///		}
+	/// 		list.Add (1);
+	/// 	}
+	/// }
+	/// 
+	/// public class OtherOverriderWithCodeDuplicated : BaseClassWithCodeDuplicated {
+	///	public void OtherMethod ()
+	///	{
+	///		foreach (int i in list) {
+	///			Console.WriteLine (i);
+	///		}
+	///		list.Remove (1);
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public class BaseClassWithoutCodeDuplicated {
+	///	protected IList list;
+	/// 
+	///	protected void PrintValuesInList ()
+	///	{
+	///		foreach (int i in list) {
+	///			Console.WriteLine (i);
+	///		}
+	///	}
+	/// }
+	/// 
+	/// public class OverriderClassWithoutCodeDuplicated : BaseClassWithoutCodeDuplicated {
+	/// 	public void SomeCode ()
+	///	{
+	///		PrintValuesInList ();
+	///		list.Add (1);
+	///	}
+	/// }
+	/// 
+	/// public class OtherOverriderWithoutCodeDuplicated : BaseClassWithoutCodeDuplicated {
+	/// 	public void MoreCode ()
+	///	{
+	///		PrintValuesInList ();
+	///		list.Remove (1);
+	///	}
+	/// }	
+	/// </code>
+	/// </example>
+
 	[Problem ("There is similar code in various methods in sibling classes.  Your code will be better if you can unify them.")]
 	[Solution ("You can apply the Pull Up Method refactoring.")]
 	public class AvoidCodeDuplicatedInSiblingClassesRule : Rule, ITypeRule {
 
-		private CodeDuplicatedLocator codeDuplicatedLocator = new CodeDuplicatedLocator ();
+		private CodeDuplicatedLocator codeDuplicatedLocator;
+
+		public AvoidCodeDuplicatedInSiblingClassesRule ()
+		{
+			codeDuplicatedLocator = new CodeDuplicatedLocator (this);
+		}
 
 		private void FindCodeDuplicated (TypeDefinition type, ICollection<TypeDefinition> siblingClasses)
 		{
-			foreach (MethodDefinition method in type.Methods)
+			foreach (MethodDefinition method in type.AllMethods ())
 				foreach (TypeDefinition sibling in siblingClasses)
-					codeDuplicatedLocator.CompareMethodAgainstTypeMethods (this, method, sibling);
+					codeDuplicatedLocator.CompareMethodAgainstTypeMethods (method, sibling);
 		}
 
 		private void CompareSiblingClasses (ICollection<TypeDefinition> siblingClasses)
@@ -58,6 +126,10 @@ namespace Gendarme.Rules.Smells {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
+			// don't analyze cases where no methods (or body) are available
+			if (type.IsEnum || type.IsInterface)
+				return RuleResult.DoesNotApply;
+
 			ICollection<TypeDefinition> siblingClasses = Utilities.GetInheritedClassesFrom (type);
 			if (siblingClasses.Count >= 2) {
 				codeDuplicatedLocator.Clear ();
