@@ -35,7 +35,45 @@ using Gendarme.Framework;
 using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Smells {
+
+	// TODO: What does "Other time, it's quite hard determine a long parameter list" mean?
+	// Can the default be changed? This rule should may need to be reviewed if C# adds
+	// default arguments. The solution is really confusing. What is "preserve whole object"?
+	// The introduction of a helper class to bundle state together? What is "parameter object"?
+	// The params keyword?
 	
+	/// <summary>
+	/// This rule allows developers to measure the parameter list size in a method.
+	/// If you have methods with a lot of parameters, perhaps you have a Long
+	/// Parameter List smell.
+	/// 
+	/// This rule counts the method's parameters, and compares it against a maximum value. 
+	/// If you have an overloaded method, then the rule will get the shortest overload 
+	/// and compare the shortest overload against the maximum value.
+	///
+	/// Other time, it's quite hard determine a long parameter list. By default, 
+	/// a method with 6 or more arguments will be flagged as a defect. 
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public void MethodWithLongParameterList (int x, char c, object obj, bool j, string f,
+	///                                         float z, double u, short s, int v, string[] array)
+	/// {
+   	/// 	// Method body ... 
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public void MethodWithoutLongParameterList (int x, object obj)
+	/// {
+	/// 	// Method body.... 
+	/// }
+	/// </code>
+	/// </example>
+
 	//SUGGESTION: Setting all required properties in a constructor isn't
 	//uncommon.
 	//SUGGESTION: Different value for public / private / protected methods *may*
@@ -56,24 +94,36 @@ namespace Gendarme.Rules.Smells {
 
 		private static MethodDefinition GetSmallerConstructorFrom (TypeDefinition type)
 		{
+			if (!type.HasConstructors)
+				return null;
+
 			if (type.Constructors.Count == 1)
 				return type.Constructors[0];
 
 			MethodDefinition smallest = null;
+			int scount = 0;
 			foreach (MethodDefinition constructor in type.Constructors) {
 				// skip the static ctor since it will always be the smallest one
 				if (constructor.IsStatic)
 					continue;
 
-				if ((smallest == null) || (smallest.Parameters.Count > constructor.Parameters.Count))
+				if (smallest == null) {
 					smallest = constructor;
+					scount = smallest.HasParameters ? smallest.Parameters.Count : 0;
+				} else {
+					int ccount = constructor.HasParameters ? constructor.Parameters.Count : 0;
+					if (scount > ccount) {
+						smallest = constructor;
+						scount = ccount;
+					}
+				}
 			}
 			return smallest;
 		}
 
 		private bool HasMoreParametersThanAllowed (MethodDefinition method)
 		{
-			return method.Parameters.Count >= MaxParameters;
+			return (method.HasParameters ? method.Parameters.Count : 0) >= MaxParameters;
 		}
 
 		private void CheckConstructor (MethodDefinition constructor)
@@ -82,9 +132,6 @@ namespace Gendarme.Rules.Smells {
 			//All stuff that doesn't contain a constructor
 			if (constructor == null) 
 				return;
-			//Skip static constructors
-			if ((constructor.Parameters.Count == 0) && !constructor.IsStatic && constructor.IsVisible ())
-				return;
 			if (HasMoreParametersThanAllowed (constructor)) 
 				Runner.Report (constructor, Severity.Medium, Confidence.Normal, "This constructor contains a long parameter list.");
 		}
@@ -106,8 +153,11 @@ namespace Gendarme.Rules.Smells {
 				if (!possibleOverloaded.ContainsKey (method.Name))
 					possibleOverloaded.Add (method.Name, method);
 				else {
-					if (possibleOverloaded[method.Name].Parameters.Count > method.Parameters.Count)
-						possibleOverloaded[method.Name] = method;
+					MethodDefinition candidate = possibleOverloaded [method.Name];
+					int ccount = candidate.HasParameters ? candidate.Parameters.Count : 0;
+					int mcount = method.HasParameters ? method.Parameters.Count : 0;
+					if (ccount > mcount)
+						possibleOverloaded [method.Name] = method;
 				}
 			}
 			return possibleOverloaded.Values;
@@ -115,13 +165,17 @@ namespace Gendarme.Rules.Smells {
 
 		private static bool OnlyContainsExternalMethods (TypeDefinition type)
 		{
+			if (!type.HasMethods)
+				return false;
+
 			foreach (MethodDefinition method in type.Methods)
 				if (!method.IsPInvokeImpl)
 					return false;
-			return type.Methods.Count != 0;
+			// all methods are p/invoke
+			return true;
 		}
 
-		private RuleResult CheckDelegate (TypeDefinition type)
+		private RuleResult CheckDelegate (TypeReference type)
 		{
 			MethodDefinition method = type.GetMethod ("Invoke");
 			// MulticastDelegate inherits from Delegate without overriding Invoke
@@ -141,9 +195,11 @@ namespace Gendarme.Rules.Smells {
 				return CheckDelegate (type);
 
 			CheckConstructor (GetSmallerConstructorFrom (type));
-			
-			foreach (MethodDefinition method in GetSmallerOverloaded (type)) 
-				CheckMethod (method);
+
+			if (type.HasMethods) {
+				foreach (MethodDefinition method in GetSmallerOverloaded (type)) 
+					CheckMethod (method);
+			}
 
 			return Runner.CurrentRuleResult;
 		}
