@@ -37,7 +37,7 @@ using Gendarme.Rules.UI;
 using NUnit.Framework;
 using Test.Rules.Helpers;
 
-namespace Test.Rules.UI {
+namespace Test.Rules.Ui {
 
 	[TestFixture]
 	public class SystemWindowsFormsExecutableTargetTest {
