@@ -27,22 +27,28 @@
 //
 
 using System;
+using System.Collections.Specialized;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.IO;
 using System.Windows.Forms;
 
 using Gendarme.Framework;
+using Gendarme.Properties;
 
 using Mono.Cecil;
+using Mono.Cecil.Binary;
 
 namespace Gendarme {
 
 	public partial class Wizard : Form {
 
-		// used to call code asynchronously
+		// This is used for methods which we execute within a thread, but we don't
+		// execute the methods concurrently so we can use single thread.
+		[ThreadModel (ThreadModel.SingleThread)]
 		delegate void MethodInvoker ();
 
+		[ThreadModel (ThreadModel.SingleThread)]
 		sealed class AssemblyInfo {
 			private DateTime timestamp;
 			private AssemblyDefinition definition;
@@ -60,6 +66,7 @@ namespace Gendarme {
 
 		private const string BaseUrl = "http://www.mono-project.com/";
 		private const string DefaultUrl = BaseUrl + "Gendarme";
+		private const string BugzillaUrl = "http://bugzilla.novell.com";
 
 		static Process process;
 
@@ -102,6 +109,7 @@ namespace Gendarme {
 			UpdatePageUI ();
 		}
 
+		[ThreadModel (ThreadModel.SingleThread)]
 		static void EndCallback (IAsyncResult result)
 		{
 			(result.AsyncState as MethodInvoker).EndInvoke (result);
@@ -151,15 +159,18 @@ namespace Gendarme {
 			case Page.SelectRules:
 				Current = Page.AddFiles;
 				break;
+			case Page.Options:
+				Current = Page.SelectRules;
+				break;
 			case Page.Analyze:
 				// then ask confirmation before aborting 
 				// and move back one step
 				if (ConfirmAnalyzeAbort (false))
-					Current = Page.SelectRules;
+					Current = Page.Options;
 				break;
 			case Page.Report:
 				// move two step back (i.e. skip analyze)
-				Current = Page.SelectRules;
+				Current = Page.Options;
 				break;
 			}
 		}
@@ -174,6 +185,10 @@ namespace Gendarme {
 				Current = Page.SelectRules;
 				break;
 			case Page.SelectRules:
+				Current = Page.Options;
+				break;
+			case Page.Options:
+				UpdateOptions ();
 				Current = Page.Analyze;
 				break;
 			case Page.Analyze:
@@ -216,6 +231,9 @@ namespace Gendarme {
 			case Page.SelectRules:
 				UpdateSelectRulesUI ();
 				break;
+			case Page.Options:
+				UpdateOptionsUI ();
+				break;
 			case Page.Analyze:
 				UpdateAnalyzeUI ();
 				break;
@@ -295,14 +313,24 @@ namespace Gendarme {
 			UpdatePageUI ();
 		}
 
+		[ThreadModel (ThreadModel.SingleThread)]
 		public void UpdateAssemblies ()
 		{
+			if (IsDisposed)
+				throw new ObjectDisposedException (GetType ().Name);
+			
 			foreach (KeyValuePair<string,AssemblyInfo> kvp in assemblies) {
 				DateTime last_write = File.GetLastWriteTimeUtc (kvp.Key);
 				if ((kvp.Value.Definition == null) || (kvp.Value.Timestamp < last_write)) {
 					AssemblyInfo a = kvp.Value;
 					a.Timestamp = last_write;
-					a.Definition = AssemblyFactory.GetAssembly (kvp.Key);
+					try {
+						a.Definition = AssemblyFactory.GetAssembly (kvp.Key);
+					}
+					catch (ImageFormatException) {
+						// continue loading & analyzing assemblies
+						// TODO: report as non-fatal warning
+					}
 				}
 			}
 		}
@@ -330,29 +358,38 @@ namespace Gendarme {
 			if (rules_populated)
 				return;
 
+			// if settings are empty (like default) then all rules are active
+			StringCollection rules = Settings.Default.Rules;
+			bool all_rules = ((rules == null) || (rules.Count == 0));
+
 			Dictionary<string, TreeNode> nodes = new Dictionary<string, TreeNode> ();
 
 			rules_tree_view.BeginUpdate ();
+			rules_tree_view.AfterCheck -= RulesTreeViewAfterCheck;
 			foreach (IRule rule in Runner.Rules) {
 				TreeNode parent;
 				string name_space = rule.FullName.Substring (0, rule.FullName.Length - rule.Name.Length - 1);
 				if (!nodes.TryGetValue (name_space, out parent)) {
 					parent = new TreeNode (name_space);
-					parent.Checked = true;
+					parent.Checked = all_rules;
 					nodes.Add (name_space, parent);
 					rules_tree_view.Nodes.Add (parent);
 				}
 
 				TreeNode node = new TreeNode (rule.Name);
-				node.Checked = true;
+				node.Checked = all_rules || rules.Contains (rule.FullName);
 				node.Tag = rule;
 				node.ToolTipText = rule.Problem;
 				parent.Nodes.Add (node);
+				// if we have not already setted parent, then we do it if any node is checked
+				if (!all_rules && node.Checked)
+					parent.Checked = true;
 			}
 			foreach (TreeNode node in rules_tree_view.Nodes) {
 				node.ToolTipText = String.Format ("{0} rules available", node.Nodes.Count);
 			}
 			nodes.Clear ();
+			rules_tree_view.AfterCheck += RulesTreeViewAfterCheck;
 
 			// this extra [End|Begin]Update is brought to you by Vista(tm)
 			// http://forums.msdn.microsoft.com/en-US/netfxbcl/thread/3fd6c4a2-b5c7-4334-b11a-e909b11e8bdc/
@@ -383,19 +420,99 @@ namespace Gendarme {
 			Open (url);
 		}
 
+		private void SaveRulesButtonClick (object sender, EventArgs e)
+		{
+			bool all_active = UpdateActiveRules ();
+			// add rule list only if they are not all active
+			if (!all_active) {
+				StringCollection rules = new StringCollection ();
+
+				foreach (IRule rule in Runner.Rules) {
+					if (rule.Active)
+						rules.Add (rule.FullName);
+				}
+				Settings.Default.Rules = rules;
+			}
+			Settings.Default.Save ();
+		}
+
 		private void RulesTreeViewAfterCheck (object sender, TreeViewEventArgs e)
 		{
-			IRule rule = (e.Node.Tag as IRule);
-			if (rule == null) {
+			if (e.Node.Tag == null) {
 				// childs
 				foreach (TreeNode node in e.Node.Nodes) {
 					node.Checked = e.Node.Checked;
 				}
-			} else {
-				rule.Active = e.Node.Checked;
 			}
 		}
 
+		[ThreadModel (ThreadModel.SingleThread)]
+		private bool UpdateActiveRules ()
+		{
+			bool all_active = true;
+			foreach (TreeNode assembly in rules_tree_view.Nodes) {
+				foreach (TreeNode rule in assembly.Nodes) {
+					if (!rule.Checked)
+						all_active = false;
+					(rule.Tag as Rule).Active = rule.Checked;
+				}
+			}
+			return all_active;
+		}
+
+		#endregion
+
+		#region Options
+
+		private void UpdateOptionsUI ()
+		{
+			options_nolimit_checkbox.Checked = (Settings.Default.DefectsLimit == Int32.MaxValue);
+
+			options_severity_combobox.SelectedIndex = (int) Severity.Audit - Settings.Default.Severity;
+			options_confidence_combobox.SelectedIndex = (int) Confidence.Low - Settings.Default.Confidence;
+
+			switch ((ApplicabilityScope) Settings.Default.Scope) {
+			case ApplicabilityScope.Visible:
+				options_visible_radiobutton.Checked = true;
+				break;
+			case ApplicabilityScope.NonVisible:
+				options_notvisible_radiobutton.Checked = true;
+				break;
+			case ApplicabilityScope.All:
+				options_all_radiobutton.Checked = true;
+				break;
+			}
+		}
+
+		private void OptionsNoLimitCheckboxCheckedChanged (object sender, EventArgs e)
+		{
+			options_limit_updown.Enabled = !options_nolimit_checkbox.Checked;
+		}
+
+		private void UpdateOptions ()
+		{
+			// 2^31 is close enough "no limit"
+			Settings.Default.DefectsLimit = options_nolimit_checkbox.Checked ? Int32.MaxValue :
+				(int) options_limit_updown.Value;
+
+			// hack, this works right now but won't if we add/remove more options
+			Settings.Default.Severity = (int) Severity.Audit - options_severity_combobox.SelectedIndex;
+			Settings.Default.Confidence = (int) Confidence.Low - options_confidence_combobox.SelectedIndex;
+
+			if (options_visible_radiobutton.Checked)
+				Settings.Default.Scope = (int) ApplicabilityScope.Visible;
+			else if (options_all_radiobutton.Checked)
+				Settings.Default.Scope = (int) ApplicabilityScope.All;
+			else
+				Settings.Default.Scope = (int) ApplicabilityScope.NonVisible;
+		}
+
+		private void OnOptionsSaveClick (object sender, EventArgs e)
+		{
+			UpdateOptions ();
+			Settings.Default.Save ();
+		}
+
 		#endregion
 
 		#region Analyze
@@ -430,17 +547,45 @@ namespace Gendarme {
 			Runner.Assemblies.Clear ();
 			foreach (KeyValuePair<string, AssemblyInfo> kvp in assemblies) {
 				// add assemblies references to runner
-				Runner.Assemblies.Add (kvp.Value.Definition);
+				AssemblyDefinition ad = kvp.Value.Definition;
+				// an invalid assembly (e.g. non-managed code) will be null at this stage
+				if (ad != null)
+					Runner.Assemblies.Add (ad);
 			}
 
 			progress_bar.Maximum = Runner.Assemblies.Count;
 		}
 
+		// TODO: Note that it isn't clear whether we can actually call gendarme from a thread...
+		[ThreadModel (ThreadModel.SingleThread)]
 		private void Analyze ()
 		{
 			counter = 0;
-			Runner.Initialize ();
-			Runner.Run ();
+
+			// apply settings
+			Runner.DefectsLimit = Settings.Default.DefectsLimit;
+
+			Runner.SeverityBitmask.ClearAll ();
+			Runner.SeverityBitmask.SetDown ((Severity)Settings.Default.Severity);
+			Runner.ConfidenceBitmask.ClearAll ();
+			Runner.ConfidenceBitmask.SetDown ((Confidence) Settings.Default.Confidence);
+
+			// wizard limits this as a "global" (all rule) setting
+			ApplicabilityScope scope = (ApplicabilityScope) Settings.Default.Scope;
+			foreach (IRule rule in Runner.Rules) {
+				rule.ApplicabilityScope = scope; 
+			}
+
+			// activate rules based on user selection
+			UpdateActiveRules ();
+
+			// Initialize / Run / TearDown
+			Runner.Execute ();
+
+			// reset rules as the user selected them (since some rules might have
+			// turned themselves off is they were not needed for the analysis) so
+			// that report used rules correctly
+			UpdateActiveRules ();
 
 			BeginInvoke ((Action) (() => Current = Page.Report));
 		}
@@ -457,7 +602,7 @@ namespace Gendarme {
 		/// Update UI before analyzing an assembly.
 		/// </summary>
 		/// <param name="e">RunnerEventArgs that contains the Assembly being analyzed and the Runner</param>
-		public void PreAssemblyUpdate (RunnerEventArgs e)
+		internal void PreAssemblyUpdate (RunnerEventArgs e)
 		{
 			progress_bar.Value = counter++;
 			analyze_status_label.Text = String.Format ("Processing assembly {0} of {1}",
@@ -469,7 +614,7 @@ namespace Gendarme {
 		/// Update UI after analyzing an assembly.
 		/// </summary>
 		/// <param name="e">RunnerEventArgs that contains the Assembly being analyzed and the Runner</param>
-		public void PostTypeUpdate (RunnerEventArgs e)
+		internal void PostTypeUpdate (RunnerEventArgs e)
 		{
 			analyze_defect_label.Text = String.Format ("Defects Found: {0}", e.Runner.Defects.Count);
 		}
@@ -487,6 +632,16 @@ namespace Gendarme {
 				has_defects ? Runner.Defects.Count.ToString () : "no");
 			cancel_button.Text = "Close";
 			next_button.Enabled = false;
+
+			// display an error message and details if we encountered an exception during analysis
+			string error = Runner.Error;
+			bool visible = (error.Length > 0);
+			unexpected_error_label.Visible = visible;
+			copy_paste_label.Visible = visible;
+			bugzilla_linklabel.Text = BugzillaUrl;
+			bugzilla_linklabel.Visible = visible;
+			error_textbox.Text = error;
+			error_textbox.Visible = visible;
 		}
 
 		private static bool CouldCopyReport (ref string currentName, string fileName)
@@ -494,8 +649,15 @@ namespace Gendarme {
 			// if possible avoid re-creating the report (as it can 
 			// be a long operation) and simply copy the file
 			bool copy = (currentName != null);
-			if (copy)
-				File.Copy (currentName, fileName);
+			if (copy) {
+				try {
+					File.Copy (currentName, fileName);
+				}
+				catch (Exception) {
+					// too many things can go wrong when copying
+					copy = false;
+				}
+			}
 
 			currentName = fileName;
 			return copy;
@@ -548,6 +710,11 @@ namespace Gendarme {
 			Open (html_report_filename);
 		}
 
+		private void BugzillaLinkClick (object sender, LinkLabelLinkClickedEventArgs e)
+		{
+			Open (BugzillaUrl);
+		}
+
 		#endregion
 	}
 }
