@@ -1,28 +1,74 @@
+//
+// CecilMetadata.cs
+//
+// (C) 2007 - 2008 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+//
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections.Generic;
-using System.Text;
-using System.Threading;
-using System.Xml;
 using System.IO;
+using System.Text;
+
 using Mono.Cecil;
 using Mono.Cecil.Cil;
-using Gtk;
 
 namespace GuiCompare {
 
 	static class CecilUtils {
-		public static string PrettyType (string type)
-		{
-			/* handle suffixes first, recursively */
-			if (type.EndsWith ("[]"))
-				return String.Concat (PrettyType (type.Substring (0, type.Length - 2)), "[]");
-			else if (type.EndsWith ("*"))
-				return String.Concat (PrettyType (type.Substring (0, type.Length - 1)), "*");
-			else if (type.EndsWith ("&"))
-				return String.Concat (PrettyType (type.Substring (0, type.Length - 1)), "&");
 
-			/* handle the system types we know about */
-			switch (type) {
+		public static string PrettyType (TypeReference type)
+		{
+			var gen_instance = type as GenericInstanceType;
+			if (gen_instance != null) {
+				if (gen_instance.ElementType.FullName == "System.Nullable`1")
+					return PrettyType (gen_instance.GenericArguments [0]) + "?";
+
+				var signature = new StringBuilder ();
+				signature.Append (PrettyType (gen_instance.ElementType));
+				signature.Append ("<");
+				for (int i = 0; i < gen_instance.GenericArguments.Count; i++) {
+					if (i > 0)
+						signature.Append (",");
+
+					signature.Append (PrettyType (gen_instance.GenericArguments [i]));
+				}
+				signature.Append (">");
+
+				return signature.ToString ();
+			}
+
+			var array = type as ArrayType;
+			if (array != null)
+				return PrettyType (array.ElementType) + "[]";
+
+			var reference = type as ReferenceType;
+			if (reference != null)
+				return PrettyType (reference.ElementType) + "&";
+
+			var pointer = type as PointerType;
+			if (pointer != null)
+				return PrettyType (pointer.ElementType) + "*";
+
+			switch (type.FullName) {
 			case "System.Boolean": return "bool";
 			case "System.Byte": return "byte";
 			case "System.Char": return "char";
@@ -41,10 +87,33 @@ namespace GuiCompare {
 			case "System.Void": return "void";
 			}
 
-			/* for other types, just return the type name */
-			return type.Substring (type.LastIndexOf ('.')+1);
+			return type.Name;
 		}
 		
+		public static string PrettyTypeDefinition (TypeDefinition td)
+		{
+			if (td.GenericParameters.Count > 0) {
+				int arity_start = td.Name.IndexOf ('`');
+				if (arity_start > 0) {
+					var sb = new StringBuilder (td.Name);
+					sb.Remove (arity_start, td.Name.Length - arity_start);
+					sb.Append ("<");
+					bool first_gp = true;
+					foreach (GenericParameter gp in td.GenericParameters) {
+						if (!first_gp)
+							sb.Append (',');
+						first_gp = false;
+						sb.Append (gp.Name);
+					}
+					sb.Append (">");
+				
+					return sb.ToString ();
+				}
+			}
+			
+			return td.Name;
+		}		
+		
 		// the corcompare xml output uses a different formatting than Cecil.
 		// Cecil uses / for nested classes, ala:
 		//  Namespace.Class/NestedClass
@@ -56,12 +125,11 @@ namespace GuiCompare {
 		//
 		// so let's just convert everything to corcompare's way of thinking for comparisons.
 		//
-		public static string FormatTypeLikeCorCompare (string type)
+		public static string FormatTypeLikeCorCompare (TypeReference type)
 		{
-			string rv = type.Replace ('/', '+');
-			rv = rv.Replace ('<', '[');
-			rv = rv.Replace ('>', ']');
-			return rv;
+			return type.FullName.Replace ('/', '+')
+				.Replace ('<', '[')
+				.Replace ('>', ']');
 		}
 		
 		public static void PopulateMemberLists (TypeDefinition fromDef,
@@ -73,31 +141,15 @@ namespace GuiCompare {
 		                                        List<CompNamed> event_list)
 		{
 			if (interface_list != null) {
-				TypeDefinition td = fromDef;
-				TypeReference fromRef;
-				
-				do {
-					foreach (TypeReference ifc in td.Interfaces) {
-						TypeDefinition ifc_def = CecilUtils.Resolver.Resolve (ifc);
-						if (ifc_def.IsNotPublic)
-							continue;
-							
-						CecilInterface new_ifc = new CecilInterface (ifc);
-						bool found = false;
-						for (int i = 0; i < interface_list.Count; i ++) {
-							if (interface_list[i].Name == new_ifc.Name) {
-								found = true;
-								break;
-							}
-						}
-						if (!found)
-							interface_list.Add (new_ifc);
-					}
-					fromRef = td.BaseType;
-					if (fromRef != null)
-						td = CecilUtils.Resolver.Resolve (fromRef);
-				} while (fromRef != null);
+				foreach (TypeReference ifc in GetInterfaces (fromDef)) {
+					TypeDefinition ifc_def = ifc.Resolve ();
+					if (ifc_def.IsNotPublic)
+						continue;
+
+					interface_list.Add (new CecilInterface (ifc));
+				}
 			}
+
 			if (constructor_list != null) {
 				foreach (MethodDefinition md in fromDef.Constructors) {
 					if (md.IsPrivate || md.IsAssembly)
@@ -111,8 +163,13 @@ namespace GuiCompare {
 						if (!md.Name.StartsWith("op_"))
 							continue;
 					}
+
+					if (IsFinalizer (md))
+						continue;
+
 					if (md.IsPrivate || md.IsAssembly)
 						continue;
+
 					method_list.Add (new CecilMethod (md));
 				}
 			}
@@ -153,6 +210,45 @@ namespace GuiCompare {
 			}
 		}
 
+		static IEnumerable<TypeDefinition> WalkHierarchy (TypeReference type)
+		{
+			for (var definition = type.Resolve (); definition != null; definition = GetBaseType (definition))
+				yield return definition;
+		}
+
+		static TypeDefinition GetBaseType (TypeDefinition type)
+		{
+			if (type.BaseType == null)
+				return null;
+
+			return type.BaseType.Resolve ();
+		}
+
+		static IEnumerable<TypeReference> GetInterfaces (TypeReference type)
+		{
+			var cache = new Dictionary<string, TypeReference> ();
+
+			foreach (var definition in WalkHierarchy (type))
+				foreach (TypeReference iface in definition.Interfaces)
+					cache [iface.FullName] = iface;
+
+			return cache.Values;
+		}
+
+		static bool IsFinalizer (MethodDefinition method)
+		{
+			if (method.Name != "Finalize")
+				return false;
+
+			if (!method.IsVirtual)
+				return false;
+
+			if (method.Parameters.Count != 0)
+				return false;
+
+			return true;
+		}
+
 		public static void PopulateTypeLists (TypeDefinition fromDef,
 		                                      List<CompNamed> class_list,
 		                                      List<CompNamed> enum_list,
@@ -177,10 +273,8 @@ namespace GuiCompare {
 				else if (type_def.IsInterface) {
 					interface_list.Add (new CecilInterface (type_def));
 				}
-				else if ((type_def.FullName == "System.MulticastDelegate" ||
-				          type_def.BaseType.FullName == "System.MulticastDelegate")
-				         || (type_def.FullName == "System.Delegate" ||
-				             type_def.BaseType.FullName == "System.Delegate")) {
+				else if (type_def.BaseType.FullName == "System.MulticastDelegate"
+				         || type_def.BaseType.FullName == "System.Delegate") {
 					delegate_list.Add (new CecilDelegate (type_def));
 				}
 				else {
@@ -214,39 +308,45 @@ namespace GuiCompare {
 			if (typedef.BaseType == null)
 				return false;
 			
-			return IsTODOAttribute (CecilUtils.Resolver.Resolve (typedef.BaseType));
-		}
-		
-		public static bool ShouldSkipAttribute (string name)
-		{
-			if (name == "System.Diagnostics.CodeAnalysis.SuppressMessageAttribute")
-				return true;
-			
-			return false;
+			return IsTODOAttribute (GetBaseType (typedef));
 		}
 		
 		public static List<CompNamed> GetCustomAttributes (ICustomAttributeProvider provider, List<string> todos)
 		{
 			List<CompNamed> rv = new List<CompNamed>();
 			foreach (CustomAttribute ca in provider.CustomAttributes) {
-				TypeDefinition resolved = CecilUtils.Resolver.Resolve (ca.Constructor.DeclaringType);
+				TypeDefinition resolved = ca.Constructor.DeclaringType.Resolve ();
 
 				if (resolved != null) {
-					if (resolved.IsNotPublic)
-						continue;
-
 					if (IsTODOAttribute (resolved)) {
 						todos.Add (String.Format ("[{0} ({1})]", ca.Constructor.DeclaringType.Name, CecilUtils.GetTODOText (ca)));					
 						continue;
 					}
+
+					if (resolved.IsNotPublic)
+						continue;
 				}
 
-				if (!ShouldSkipAttribute (ca.Constructor.DeclaringType.FullName))
+				if (!MasterUtils.IsImplementationSpecificAttribute (ca.Constructor.DeclaringType.FullName))
 					rv.Add (new CecilAttribute (ca));
 			}
 			return rv;
 		}
-
+		
+		public static List<CompGenericParameter> GetTypeParameters (IGenericParameterProvider provider)
+		{
+			if (provider.GenericParameters.Count == 0)
+				return null;
+				
+			var l = new List<CompGenericParameter> ();
+			
+			var gparameters = provider.GenericParameters;
+			foreach (GenericParameter gp in gparameters) {
+				l.Add (new CecilGenericParameter (gp));
+			}			
+			
+			return l;			
+		}
 		
 		public static readonly AssemblyResolver Resolver = new AssemblyResolver();
 	}
@@ -255,9 +355,9 @@ namespace GuiCompare {
 		public CecilAssembly (string path)
 			: base (Path.GetFileName (path))
 		{
-			Dictionary<string, Dictionary <string, TypeDefinition>> namespaces = new Dictionary<string, Dictionary <string, TypeDefinition>> ();
+			var namespaces = new Dictionary<string, Dictionary<string, TypeDefinition>> ();
 
-			AssemblyDefinition assembly = AssemblyFactory.GetAssembly(path);
+			var assembly = AssemblyFactory.GetAssembly(path);
 
 			foreach (TypeDefinition t in assembly.MainModule.Types) {
 				if (t.Name == "<Module>")
@@ -265,27 +365,30 @@ namespace GuiCompare {
 
 				if (t.IsNotPublic)
 					continue;
-				
+
 				if (t.IsNested)
 					continue;
-				
+
 				if (t.IsSpecialName || t.IsRuntimeSpecialName)
 					continue;
 
 				if (CecilUtils.IsTODOAttribute (t))
 					continue;
 
-				if (!namespaces.ContainsKey (t.Namespace))
-					namespaces[t.Namespace] = new Dictionary <string, TypeDefinition> ();
+				Dictionary<string, TypeDefinition> ns;
 
-				namespaces[t.Namespace][t.Name] = t;
+				if (!namespaces.TryGetValue (t.Namespace, out ns)) {
+					ns = new Dictionary<string, TypeDefinition> ();
+					namespaces.Add (t.Namespace, ns);
+				}
+
+				ns [t.Name] = t;
 			}
 
-			namespace_list = new List<CompNamed>();
-			foreach (string ns_name in namespaces.Keys) {
-				namespace_list.Add (new CecilNamespace (ns_name, namespaces[ns_name]));
-			}
-			
+			namespace_list = new List<CompNamed> ();
+			foreach (string ns_name in namespaces.Keys)
+				namespace_list.Add (new CecilNamespace (ns_name, namespaces [ns_name]));
+
 			attributes = CecilUtils.GetCustomAttributes (assembly, todos);
 		}
 
@@ -312,7 +415,8 @@ namespace GuiCompare {
  			delegate_list = new List<CompNamed>();
 			interface_list = new List<CompNamed>();
 			struct_list = new List<CompNamed>();
-
+			MemberName = name;
+			
 			foreach (string type_name in type_mapping.Keys) {
 				TypeDefinition type_def = type_mapping[type_name];
 				if (type_def.IsNotPublic)
@@ -331,10 +435,8 @@ namespace GuiCompare {
 				else if (type_def.IsInterface) {
 					interface_list.Add (new CecilInterface (type_def));
 				}
-				else if ((type_def.FullName == "System.MulticastDelegate" ||
-				          (type_def.BaseType != null && type_def.BaseType.FullName == "System.MulticastDelegate"))
-				         || (type_def.FullName == "System.Delegate" ||
-				             (type_def.BaseType != null && type_def.BaseType.FullName == "System.Delegate"))) {
+				else if ((type_def.BaseType != null && type_def.BaseType.FullName == "System.MulticastDelegate")
+				         || (type_def.BaseType != null && type_def.BaseType.FullName == "System.Delegate")) {
 					delegate_list.Add (new CecilDelegate (type_def));
 				}
 				else {
@@ -377,9 +479,10 @@ namespace GuiCompare {
 
 	public class CecilInterface : CompInterface {		
 		public CecilInterface (TypeDefinition type_def)
-			: base (CecilUtils.FormatTypeLikeCorCompare (type_def.Name))
+			: base (type_def.Name)
 		{
 			this.type_def = type_def;
+			DisplayName = CecilUtils.PrettyTypeDefinition (type_def);
 			
 			interfaces = new List<CompNamed>();
 			constructors = new List<CompNamed>();
@@ -387,7 +490,10 @@ namespace GuiCompare {
 			properties = new List<CompNamed>();
 			fields = new List<CompNamed>();
 			events = new List<CompNamed>();
-
+			if (!type_def.IsNotPublic || type_def.IsPublic || type_def.IsNestedPublic || type_def.IsNestedFamily ||
+			    type_def.IsNestedFamilyAndAssembly || type_def.IsNestedFamilyOrAssembly)
+				MemberName = type_def.FullName;
+			
 			CecilUtils.PopulateMemberLists (type_def,
 			                                interfaces,
 			                                constructors,
@@ -397,10 +503,11 @@ namespace GuiCompare {
 			                                events);
 			
 			attributes = CecilUtils.GetCustomAttributes (type_def, todos);
+			tparams = CecilUtils.GetTypeParameters (type_def);
 		}
-		
+
 		public CecilInterface (TypeReference type_ref)
-			: base (CecilUtils.FormatTypeLikeCorCompare (type_ref.FullName))
+			: base (CecilUtils.FormatTypeLikeCorCompare (type_ref))
 		{
 			interfaces = new List<CompNamed>();
 			constructors = new List<CompNamed>();
@@ -410,11 +517,12 @@ namespace GuiCompare {
 			events = new List<CompNamed>();
 			
 			attributes = new List<CompNamed>();
+			tparams = new List<CompGenericParameter>();
 		}
 
 		public override string GetBaseType ()
 		{
-			return (type_def == null || type_def.BaseType == null) ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType.FullName);
+			return (type_def == null || type_def.BaseType == null) ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType);
 		}
 		
 		public override List<CompNamed> GetInterfaces ()
@@ -452,6 +560,11 @@ namespace GuiCompare {
 			return attributes;
 		}
 		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return tparams;
+		}
+		
 		List<CompNamed> interfaces;
 		List<CompNamed> constructors;
 		List<CompNamed> methods;
@@ -459,6 +572,7 @@ namespace GuiCompare {
 		List<CompNamed> fields;
 		List<CompNamed> events;
 		List<CompNamed> attributes;
+		List<CompGenericParameter> tparams;
 		TypeDefinition type_def;
 	}
 
@@ -467,14 +581,32 @@ namespace GuiCompare {
 			: base (type_def.Name)
 		{
 			this.type_def = type_def;
+			DisplayName = CecilUtils.PrettyTypeDefinition (type_def);
+
+			if (!type_def.IsNotPublic || type_def.IsPublic || type_def.IsNestedPublic || type_def.IsNestedFamily ||
+			    type_def.IsNestedFamilyAndAssembly || type_def.IsNestedFamilyOrAssembly)
+				MemberName = type_def.FullName;
+				
+			attributes = CecilUtils.GetCustomAttributes (type_def, todos);				
+		}
+		
+		public override List<CompNamed> GetAttributes ()
+		{
+			return attributes;
 		}
 
 		public override string GetBaseType ()
 		{
-			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType.FullName);
+			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType);
+		}
+		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return CecilUtils.GetTypeParameters (type_def);
 		}
 		
 		TypeDefinition type_def;
+		List<CompNamed> attributes;
 	}
 
 	public class CecilEnum : CompEnum {
@@ -484,7 +616,10 @@ namespace GuiCompare {
 			this.type_def = type_def;
 
 			fields = new List<CompNamed>();
-
+			if (!type_def.IsNotPublic || type_def.IsPublic || type_def.IsNestedPublic || type_def.IsNestedFamily ||
+			    type_def.IsNestedFamilyAndAssembly || type_def.IsNestedFamilyOrAssembly)
+				MemberName = type_def.FullName;
+			
 			CecilUtils.PopulateMemberLists (type_def,
 						   null,
 						   null,
@@ -493,12 +628,12 @@ namespace GuiCompare {
 						   fields,
 						   null);
 			
-			attributes = CecilUtils.GetCustomAttributes (type_def, todos); 
+			attributes = CecilUtils.GetCustomAttributes (type_def, todos);
 		}
 
 		public override string GetBaseType ()
 		{
-			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType.FullName);
+			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType);
 		}
 
  		public override List<CompNamed> GetFields()
@@ -521,6 +656,7 @@ namespace GuiCompare {
 			: base (type_def.Name, type)
 		{
 			this.type_def = type_def;
+			DisplayName = CecilUtils.PrettyTypeDefinition (type_def);
 
 			nested_classes = new List<CompNamed>();
 			nested_enums = new List<CompNamed>();
@@ -542,6 +678,10 @@ namespace GuiCompare {
 			fields = new List<CompNamed>();
 			events = new List<CompNamed>();
 
+			if (!type_def.IsNotPublic || type_def.IsPublic || type_def.IsNestedPublic || type_def.IsNestedFamily ||
+			    type_def.IsNestedFamilyAndAssembly || type_def.IsNestedFamilyOrAssembly)
+				MemberName = type_def.FullName;
+			
 			CecilUtils.PopulateMemberLists (type_def,
 			                           interfaces,
 			                           constructors,
@@ -555,8 +695,11 @@ namespace GuiCompare {
 
 		public override string GetBaseType ()
 		{
-			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType.FullName);
+			return type_def.BaseType == null ? null : CecilUtils.FormatTypeLikeCorCompare (type_def.BaseType);
 		}
+		
+		public override bool IsAbstract { get { return type_def.IsAbstract; } }
+		public override bool IsSealed { get { return type_def.IsSealed; } }
 
 		public override List<CompNamed> GetInterfaces ()
 		{
@@ -617,6 +760,11 @@ namespace GuiCompare {
 		{
 			return nested_delegates;
 		}
+		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return CecilUtils.GetTypeParameters (type_def);
+		}
 
 		TypeDefinition type_def;
 		List<CompNamed> nested_classes;
@@ -640,11 +788,16 @@ namespace GuiCompare {
 		{
 			this.field_def = field_def;
 			this.attributes = CecilUtils.GetCustomAttributes (field_def, todos);
+			if (field_def.IsPublic || field_def.IsFamily || field_def.IsFamilyAndAssembly || field_def.IsFamilyOrAssembly) {
+				TypeDefinition declType = field_def.DeclaringType;
+				if (declType != null)
+					MemberName = declType.FullName + "." + field_def.Name;
+			}
 		}
 
 		public override string GetMemberType ()
 		{
-			return CecilUtils.FormatTypeLikeCorCompare (field_def.FieldType.FullName);
+			return CecilUtils.FormatTypeLikeCorCompare (field_def.FieldType);
 		}
 		
 		const FieldAttributes masterInfoFieldMask = (FieldAttributes.FieldAccessMask | 
@@ -652,7 +805,8 @@ namespace GuiCompare {
 		                                             FieldAttributes.InitOnly | 
 		                                             FieldAttributes.Literal | 
 		                                             FieldAttributes.HasDefault | 
-		                                             FieldAttributes.HasFieldMarshal);
+		                                             FieldAttributes.HasFieldMarshal |
+		                                             FieldAttributes.NotSerialized );
 		public override string GetMemberAccess ()
 		{
 			FieldAttributes fa = field_def.Attributes & masterInfoFieldMask;
@@ -671,7 +825,7 @@ namespace GuiCompare {
 
 		public override string GetLiteralValue ()
 		{
-			if (field_def.IsLiteral)
+			if (field_def.IsLiteral && field_def.Constant != null)
 				return field_def.Constant.ToString();
 			return null;
 		}
@@ -687,6 +841,11 @@ namespace GuiCompare {
 			this.method_def = method_def;
 			this.attributes = CecilUtils.GetCustomAttributes (method_def, todos);
 			DisplayName = FormatName (method_def, true);
+			if (method_def.IsFamily || method_def.IsFamilyAndAssembly || method_def.IsFamilyOrAssembly || method_def.IsPublic) {
+				TypeReference declType = method_def.DeclaringType;
+				if (declType != null)
+					MemberName = declType.FullName + "." + method_def.Name;
+			}
 		}
 
 		public override string GetMemberType ()
@@ -694,7 +853,7 @@ namespace GuiCompare {
 			if (method_def.IsConstructor)
 				return null;
 			
-			return CecilUtils.FormatTypeLikeCorCompare (method_def.ReturnType.ReturnType.FullName);
+			return CecilUtils.FormatTypeLikeCorCompare (method_def.ReturnType.ReturnType);
 		}
 
 		public override bool ThrowsNotImplementedException ()
@@ -730,19 +889,19 @@ namespace GuiCompare {
 		{
 			return attributes;
 		}
-
-		void SetDisplayName ()
+		
+		public override List<CompGenericParameter> GetTypeParameters ()
 		{
-
-		}
+			return CecilUtils.GetTypeParameters (method_def);
+		}	
 		
 		static string FormatName (MethodDefinition method_def, bool beautify)
 		{
 			StringBuilder sb = new StringBuilder ();
 			if (!method_def.IsConstructor)
 				sb.Append (beautify
-				           ? CecilUtils.PrettyType (method_def.ReturnType.ReturnType.FullName)
-				           : CecilUtils.FormatTypeLikeCorCompare (method_def.ReturnType.ReturnType.FullName));
+				           ? CecilUtils.PrettyType (method_def.ReturnType.ReturnType)
+				           : CecilUtils.FormatTypeLikeCorCompare (method_def.ReturnType.ReturnType));
 			sb.Append (" ");
 			if (beautify) {
 				if (method_def.IsSpecialName && method_def.Name.StartsWith ("op_")) {
@@ -764,6 +923,13 @@ namespace GuiCompare {
 					case "op_UnaryPlus": sb.Append ("operator +"); break;
 					case "op_Decrement": sb.Append ("operator --"); break;
 					case "op_Increment": sb.Append ("operator ++"); break;
+					case "op_BitwiseAnd": sb.Append ("operator &"); break;
+					case "op_BitwiseOr": sb.Append ("operator |"); break;
+					case "op_ExclusiveOr": sb.Append ("operator ^"); break;
+					case "op_LogicalNot": sb.Append ("operator !"); break;
+					case "op_OnesComplement": sb.Append ("operator ~"); break;
+					case "op_True": sb.Append ("operator true"); break;
+					case "op_False": sb.Append ("operator false"); break;
 					default: Console.WriteLine ("unhandled operator named {0}", method_def.Name); sb.Append (method_def.Name); break;
 					}
 				}
@@ -796,8 +962,8 @@ namespace GuiCompare {
 				else if (p.IsOut)
 					sb.Append ("out ");
 				sb.Append (beautify
-				           ? CecilUtils.PrettyType (p.ParameterType.FullName)
-				           : CecilUtils.FormatTypeLikeCorCompare (p.ParameterType.FullName));
+				           ? CecilUtils.PrettyType (p.ParameterType)
+				           : CecilUtils.FormatTypeLikeCorCompare (p.ParameterType));
 				if (beautify) {
 					sb.Append (" ");
 					sb.Append (p.Name);
@@ -820,11 +986,27 @@ namespace GuiCompare {
 			this.pd = pd;
 			this.attributes = CecilUtils.GetCustomAttributes (pd, todos);
 			this.DisplayName = FormatName (pd, true);
+
+			MethodDefinition getMethod = pd.GetMethod, setMethod = pd.SetMethod;			
+			if (getMethod != null || setMethod != null) {
+				bool interesting = false;
+
+				if (getMethod != null && (getMethod.IsPublic || getMethod.IsFamily || getMethod.IsFamilyAndAssembly || getMethod.IsFamilyOrAssembly))
+					interesting = true;
+				else if (setMethod != null && (setMethod.IsPublic || setMethod.IsFamily || setMethod.IsFamilyAndAssembly || setMethod.IsFamilyOrAssembly))
+					interesting = true;
+
+				if (interesting) {
+					TypeDefinition declType = pd.DeclaringType;
+					if (declType != null)
+						MemberName = declType.FullName + "." + pd.Name;
+				}
+			}
 		}
 
 		public override string GetMemberType()
 		{
-			return CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType.FullName);
+			return CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType);
 		}
 		
 		public override string GetMemberAccess()
@@ -855,12 +1037,12 @@ namespace GuiCompare {
 
 #if INCLUDE_TYPE_IN_PROPERTY_DISPLAYNAME
 			sb.Append (beautify
-				           ? CecilUtils.PrettyType (pd.PropertyType.FullName)
-				           : CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType.FullName));
+				           ? CecilUtils.PrettyType (pd.PropertyType)
+				           : CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType));
 			sb.Append (" ");
 #else
 			if (!beautify) {
-				sb.Append (CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType.FullName));
+				sb.Append (CecilUtils.FormatTypeLikeCorCompare (pd.PropertyType));
 				sb.Append (" ");
 			}
 #endif
@@ -874,8 +1056,8 @@ namespace GuiCompare {
 						sb.Append (", ");
 					first_p = false;
 					sb.Append (beautify
-						   ? CecilUtils.PrettyType (p.ParameterType.FullName)
-						   : CecilUtils.FormatTypeLikeCorCompare (p.ParameterType.FullName));
+						   ? CecilUtils.PrettyType (p.ParameterType)
+						   : CecilUtils.FormatTypeLikeCorCompare (p.ParameterType));
 					if (beautify) {
 						sb.Append (" ");
 						sb.Append (p.Name);
@@ -898,11 +1080,27 @@ namespace GuiCompare {
 		{
 			this.ed = ed;
 			this.attributes = CecilUtils.GetCustomAttributes (ed, todos);
+
+			MethodDefinition addMethod = ed.AddMethod, removeMethod = ed.RemoveMethod;
+			if (addMethod != null || removeMethod != null) {
+				bool interesting = false;
+
+				if (addMethod != null && (addMethod.IsPublic || addMethod.IsFamily || addMethod.IsFamilyAndAssembly || addMethod.IsFamilyOrAssembly))
+					interesting = true;
+				else if (removeMethod != null && (removeMethod.IsPublic || removeMethod.IsFamily || removeMethod.IsFamilyAndAssembly || removeMethod.IsFamilyOrAssembly))
+					interesting = true;
+
+				if (interesting) {
+					TypeDefinition declType = ed.DeclaringType;
+					if (declType != null)
+						MemberName = declType.FullName + "." + ed.Name;
+				}
+			}
 		}
 
 		public override string GetMemberType()
 		{
-			return CecilUtils.FormatTypeLikeCorCompare (ed.EventType.FullName);
+			return CecilUtils.FormatTypeLikeCorCompare (ed.EventType);
 		}
 		
 		public override string GetMemberAccess()
@@ -924,9 +1122,28 @@ namespace GuiCompare {
 		public CecilAttribute (CustomAttribute ca)
 			: base (ca.Constructor.DeclaringType.FullName)
 		{
-			this.ca = ca;
+		}
+	}
+	
+	public class CecilGenericParameter : CompGenericParameter
+	{
+		List<CompNamed> attributes;		
+		
+		public CecilGenericParameter (GenericParameter gp)
+			: base (gp.Name, gp.Attributes)
+		{
+			attributes = CecilUtils.GetCustomAttributes (gp, todos);
+			
+			var constraints = gp.Constraints;
+			if (constraints.Count == 0)
+				return;
+				
+			// TODO: finish constraints loading
 		}
 		
-		CustomAttribute ca;
+		public override List<CompNamed> GetAttributes ()
+		{
+			return attributes;
+		}
 	}
-}
\ No newline at end of file
+}
