@@ -69,30 +69,34 @@ namespace GuiCompare {
 
 		void CompareThread ()
 		{
-			ProgressChange (Double.NaN, "Loading reference...");
+			try {
+				ProgressChange (Double.NaN, "Loading reference...");
 
-			if (!TryLoad (ref reference, reference_loader))
-				return;
+				if (!TryLoad (ref reference, reference_loader))
+					return;
 
-			ProgressChange (Double.NaN, "Loading target...");
+				ProgressChange (Double.NaN, "Loading target...");
 
-			if (!TryLoad (ref target, target_loader))
-				return;
+				if (!TryLoad (ref target, target_loader))
+					return;
 
-			ProgressChange (0.0, "Comparing...");
+				ProgressChange (0.0, "Comparing...");
 
-			comparison = target.GetComparisonNode ();
+				comparison = target.GetComparisonNode ();
 
-			List<CompNamed> ref_namespaces = reference.GetNamespaces();
-			
-			total_comparisons = CountComparisons (ref_namespaces);
-			comparisons_performed = 0;
-			
-			CompareTypeLists (comparison, reference.GetNamespaces(), target.GetNamespaces());
+				List<CompNamed> ref_namespaces = reference.GetNamespaces();
+				
+				total_comparisons = CountComparisons (ref_namespaces);
+				comparisons_performed = 0;
+				
+				CompareTypeLists (comparison, reference.GetNamespaces(), target.GetNamespaces());
 
-			CompareAttributes (comparison, reference, target);
-
-			Finish ();
+				CompareAttributes (comparison, reference, target);
+			} catch (Exception exc) {
+				OnError (exc.Message);
+			} finally {
+				Finish ();
+			}
 		}
 
 		int total_comparisons;
@@ -153,6 +157,51 @@ namespace GuiCompare {
 								reference_type.GetBaseType(),
 								target_type.GetBaseType()));
 			}
+			
+			if (reference_type.IsAbstract != target_type.IsAbstract) {
+				string ref_mod = (reference_type.IsAbstract && reference_type.IsSealed) ? "static" : "abstract";
+				string tar_mod = (target_type.IsAbstract && target_type.IsSealed) ? "static" : "abstract";
+
+				parent.AddError (String.Format ("reference is {0} {2}, target is {1} {3}",
+								reference_type.IsAbstract ? null : "not", target_type.IsAbstract ? null : "not",
+								ref_mod, tar_mod));
+			} else if (reference_type.IsSealed != target_type.IsSealed) {
+				string ref_mod = (reference_type.IsAbstract && reference_type.IsSealed) ? "static" : "sealed";
+				string tar_mod = (target_type.IsAbstract && target_type.IsSealed) ? "static" : "sealed";
+				
+				parent.AddError (String.Format ("reference is {0} {2}, target is {1} {3}",
+								reference_type.IsSealed ? null : "not", target_type.IsSealed ? null : "not",
+								ref_mod, tar_mod));
+			}
+		}
+		
+		void CompareTypeParameters (ComparisonNode parent, ICompGenericParameter reference, ICompGenericParameter target)
+		{
+			var r = reference.GetTypeParameters ();
+			var t = target.GetTypeParameters ();
+			if (r == null && t == null)
+				return;
+
+			if (r.Count != t.Count) {
+				throw new NotImplementedException (string.Format ("Should never happen with valid data ({0} != {1})", r.Count, t.Count));
+			}
+
+			for (int i = 0; i < r.Count; ++i) {
+				var r_i = r [i];
+				var t_i = t [i];
+				
+				if (r_i.GenericAttributes != t_i.GenericAttributes) {
+					parent.AddError (string.Format ("reference type parameter {2} has {0} generic attributes, target type parameter {3} has {1} generic attributes",
+						CompGenericParameter.GetGenericAttributeDesc (r_i.GenericAttributes),
+						CompGenericParameter.GetGenericAttributeDesc (t_i.GenericAttributes),
+						r_i.Name,
+						t_i.Name));
+				}
+
+				CompareAttributes (parent, r_i, t_i);
+				
+				// TODO: Compare contraints
+			}
 		}
 
 		void CompareTypeLists (ComparisonNode parent,
@@ -193,6 +242,13 @@ namespace GuiCompare {
 								  (ICompHasBaseType)target_list[a]);
 					}
 					
+					// compares generic type parameters
+					if (reference_list[m] is ICompGenericParameter && target_list[a] is ICompGenericParameter) {
+						CompareTypeParameters (comparison,
+								(ICompGenericParameter)reference_list[m],
+								(ICompGenericParameter)target_list[a]);
+					}
+					
 					// compare nested types
 					if (reference_list[m] is ICompTypeContainer && target_list[a] is ICompTypeContainer) {
 						CompareNestedTypes (comparison,
@@ -427,7 +483,8 @@ namespace GuiCompare {
 				if (!string.IsNullOrEmpty (baseTypeName)) {
 					ComparisonNode baseTypeNode = new ComparisonNode (CompType.Class,
 											  string.Format ("BaseType: {0}",
-													 baseTypeName));
+													 baseTypeName),
+											  baseTypeName);
 					baseTypeNode.Status = ComparisonStatus.Missing;
 					node.AddChild (baseTypeNode);
 				}
