@@ -1,10 +1,31 @@
+//
+// MasterMetadata.cs
+//
+// (C) 2007 - 2008 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+//
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Collections.Generic;
-using System.Threading;
 using System.Text;
-using System.Xml;
-using Mono.Cecil;
-using Gtk;
 
 namespace GuiCompare {
 
@@ -19,8 +40,8 @@ namespace GuiCompare {
 
 				XMLParameters parameters = (methods.parameters == null ? null
 				                            : (XMLParameters)methods.parameters[key]);
-				XMLGenericMethodConstraints genericConstraints = (methods.genericConstraints == null ? null
-				                                                  : (XMLGenericMethodConstraints)methods.genericConstraints[key]);
+				XMLGenericParameters genericParameters = (methods.genericParameters == null ? null
+				                                                  : (XMLGenericParameters)methods.genericParameters[key]);
 				XMLAttributes attributes = (methods.attributeMap == null ? null
 				                            : (XMLAttributes)methods.attributeMap[key]);
 				string returnType = (methods.returnTypes == null ? null
@@ -29,7 +50,7 @@ namespace GuiCompare {
 				                                   signatureFlags,
 				                                   returnType,
 				                                   parameters,
-				                                   genericConstraints,
+				                                   genericParameters,
 				                                   methods.ConvertToString (Int32.Parse ((string)methods.access[key])),
 				                                   attributes));
 			}
@@ -120,11 +141,25 @@ namespace GuiCompare {
 			}
 		}
 		
-		public static bool ShouldSkipAttribute (string name)
+		public static bool IsImplementationSpecificAttribute (string name)
 		{
-			if (name.StartsWith ("System.Diagnostics.CodeAnalysis.SuppressMessageAttribute"))
+			if (name.IndexOf (';') > 0) {
+				if (name.StartsWith ("System.Runtime.CompilerServices.InternalsVisibleToAttribute"))
+					return true;
+			}
+		
+			switch (name) {
+				case "System.Diagnostics.CodeAnalysis.SuppressMessageAttribute":
+				case "System.NonSerializedAttribute":
+			    	case "System.Runtime.CompilerServices.CompilerGeneratedAttribute":
+				case "System.Security.SecuritySafeCriticalAttribute":
+				case "System.Security.SecurityCriticalAttribute":
+				case "System.Diagnostics.DebuggerHiddenAttribute":
+				case "System.Diagnostics.DebuggerStepThroughAttribute":
+				case "System.Runtime.CompilerServices.InternalsVisibleToAttribute":
 				return true;
-			
+			}
+				
 			return false;
 		}
 		
@@ -132,8 +167,8 @@ namespace GuiCompare {
 		{
 			List<CompNamed> rv = new List<CompNamed>();
 			if (attributes != null) {
-				foreach (object key in attributes.keys.Keys) {
-					if (ShouldSkipAttribute ((string)key))
+				foreach (string key in attributes.keys.Keys) {
+					if (IsImplementationSpecificAttribute (key))
 						continue;
 					rv.Add (new MasterAttribute ((string)attributes.keys[key]));
 				}
@@ -141,6 +176,23 @@ namespace GuiCompare {
 			return rv;
 		}
 
+		public static List<CompGenericParameter> GetTypeParameters (XMLGenericParameters gparams)
+		{
+			if (gparams == null)
+				return null;
+
+			var list = new List<CompGenericParameter> ();
+			foreach (string key in gparams.keys.Keys) {
+				XMLAttributes attributes = gparams.attributeMap == null ?
+					null : (XMLAttributes) gparams.attributeMap [key];
+					
+				var constraints = gparams.constraints [key];
+				
+				list.Add (new MasterGenericTypeParameter (key, constraints.attributes, attributes));
+			}
+			
+			return list;
+		}
 	}
 	
 	public class MasterAssembly : CompAssembly {
@@ -148,6 +200,8 @@ namespace GuiCompare {
 			: base (path)
 		{
 			masterinfo = XMLAssembly.CreateFromFile (path);
+			if (masterinfo == null)
+				throw new ArgumentException ("Error loading masterinfo from " + path);
 			attributes = MasterUtils.GetAttributes (masterinfo.attributes);
 		}
 
@@ -306,6 +360,11 @@ namespace GuiCompare {
 			return attributes;
 		}
 		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return xml_cls.GetTypeParameters ();
+		}
+		
 		XMLClass xml_cls;
 		List<CompNamed> interfaces;
 		List<CompNamed> constructors;
@@ -322,12 +381,22 @@ namespace GuiCompare {
 		{
 			xml_cls = cls;
 		}
+		
+		public override List<CompNamed> GetAttributes ()
+		{
+			return MasterUtils.GetAttributes (xml_cls.attributes);
+		}
 
 		public override string GetBaseType ()
 		{
 			return xml_cls.baseName;
 		}
 		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return xml_cls.GetTypeParameters ();
+		}
+		
 		XMLClass xml_cls;
 	}
 
@@ -408,6 +477,9 @@ namespace GuiCompare {
 			return xml_cls.baseName;
 		}
 		
+		public override bool IsSealed { get { return xml_cls.isSealed; } }
+		public override bool IsAbstract { get { return xml_cls.isAbstract; } }		
+		
 		public override List<CompNamed> GetInterfaces ()
 		{
 			return interfaces;
@@ -467,6 +539,11 @@ namespace GuiCompare {
 		{
 			return delegate_list;
 		}
+		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return xml_cls.GetTypeParameters ();
+		}
 
 		XMLClass xml_cls;
 
@@ -527,7 +604,8 @@ namespace GuiCompare {
 		{
 			this.fieldType = fieldType;
 			this.fieldValue = fieldValue;
-			this.fieldAccess = fieldAccess;
+			// we don't care about the Assembly (internal) part
+			this.fieldAccess = fieldAccess.Replace ("FamORAssem", "Family");
 			this.attributes = attributes;
 		}
 
@@ -618,7 +696,7 @@ namespace GuiCompare {
 		                     XMLMethods.SignatureFlags signatureFlags,
 		                     string returnType,
 		                     XMLParameters parameters,
-		                     XMLGenericMethodConstraints genericConstraints,
+		                     XMLGenericParameters genericParameters,
 		                     string methodAccess,
 		                     XMLAttributes attributes)
 			: base (String.Format ("{0} {1}", returnType, name))
@@ -626,8 +704,9 @@ namespace GuiCompare {
 			this.signatureFlags = signatureFlags;
 			this.returnType = returnType;
 			this.parameters = parameters;
-			this.genericConstraints = genericConstraints;
-			this.methodAccess = methodAccess;
+			this.genericParameters = genericParameters;
+			// we don't care about the Assembly (internal) part
+			this.methodAccess = methodAccess.Replace ("FamORAssem", "Family");
 			this.attributes = attributes;
 		}
 
@@ -650,11 +729,16 @@ namespace GuiCompare {
 		{
 			return MasterUtils.GetAttributes (attributes);
 		}
+		
+		public override List<CompGenericParameter> GetTypeParameters ()
+		{
+			return MasterUtils.GetTypeParameters (genericParameters);
+		}
 
 		XMLMethods.SignatureFlags signatureFlags;
 		string returnType;
 		XMLParameters parameters;
-		XMLGenericMethodConstraints genericConstraints;
+		XMLGenericParameters genericParameters;
 		string methodAccess;
 		XMLAttributes attributes;
 	}
@@ -665,4 +749,19 @@ namespace GuiCompare {
 		{
 		}
 	}
+	
+	public class MasterGenericTypeParameter : CompGenericParameter {
+		XMLAttributes attributes;
+		
+		public MasterGenericTypeParameter (string name, string genericAttribute, XMLAttributes attributes)
+			: base (name, (Mono.Cecil.GenericParameterAttributes)Enum.Parse (typeof (Mono.Cecil.GenericParameterAttributes), genericAttribute))
+		{
+			this.attributes = attributes;
+		}
+		
+		public override List<CompNamed> GetAttributes ()
+		{
+			return MasterUtils.GetAttributes (attributes);
+		}
+	}
 }
